@@ -1,8 +1,9 @@
 from .composite_solid import composite_solid
+from .graph import graph
 from .hook import failure_hook, success_hook
-from .lambda_solid import lambda_solid
+from .op import op
 from .pipeline import pipeline
 from .repository import repository
 from .schedule import daily_schedule, hourly_schedule, monthly_schedule, schedule, weekly_schedule
-from .solid import solid
-from .trigger import triggered_execution
+from .sensor import sensor
+from .solid import lambda_solid, solid
