@@ -1,22 +1,48 @@
+from typing import Callable, Dict, Generic, List, Optional, Type, TypeVar, Union, cast
+
 from dagster import check
 from dagster.core.errors import DagsterInvalidDefinitionError, DagsterInvariantViolationError
 from dagster.utils import merge_dicts
 
+from .graph import GraphDefinition
 from .partition import PartitionScheduleDefinition, PartitionSetDefinition
 from .pipeline import PipelineDefinition
 from .schedule import ScheduleDefinition
-from .trigger import TriggeredExecutionDefinition
+from .sensor import SensorDefinition
+from .utils import check_valid_name
 
 VALID_REPOSITORY_DATA_DICT_KEYS = {
     "pipelines",
     "partition_sets",
     "schedules",
-    "triggered_executions",
+    "sensors",
 }
 
+RepositoryLevelDefinition = TypeVar(
+    "RepositoryLevelDefinition", PipelineDefinition, PartitionSetDefinition, ScheduleDefinition
+)
 
-class _CacheingDefinitionIndex(object):
-    def __init__(self, definition_class, definition_class_name, definition_kind, definitions):
+
+class _CacheingDefinitionIndex(Generic[RepositoryLevelDefinition]):
+    def __init__(
+        self,
+        definition_class: Type[RepositoryLevelDefinition],
+        definition_class_name: str,
+        definition_kind: str,
+        definitions: Dict[
+            str, Union[RepositoryLevelDefinition, Callable[[], RepositoryLevelDefinition]]
+        ],
+        validation_fn: Callable[[RepositoryLevelDefinition], RepositoryLevelDefinition],
+        lazy_definitions_fn: Optional[Callable[[], List[RepositoryLevelDefinition]]] = None,
+    ):
+        """
+        Args:
+            definitions: A dictionary of definition names to definitions or functions that load
+                definitions.
+            lazy_definitions_fn: A function for loading a list of definitions whose names are not
+                even known until loaded.
+
+        """
 
         for key, definition in definitions.items():
             check.invariant(
@@ -30,28 +56,49 @@ class _CacheingDefinitionIndex(object):
                 ),
             )
 
-        self._definition_class = definition_class
+        self._definition_class: Type[RepositoryLevelDefinition] = definition_class
         self._definition_class_name = definition_class_name
         self._definition_kind = definition_kind
+        self._validation_fn: Callable[
+            [RepositoryLevelDefinition], RepositoryLevelDefinition
+        ] = validation_fn
 
-        self._definitions = definitions
-        self._definition_cache = {}
-        self._definition_names = None
-        self._all_definitions = None
+        self._definitions: Dict[
+            str, Union[RepositoryLevelDefinition, Callable[[], RepositoryLevelDefinition]]
+        ] = definitions
+        self._definition_cache: Dict[str, RepositoryLevelDefinition] = {}
+        self._definition_names: Optional[List[str]] = None
 
-    def get_definition_names(self):
+        self._lazy_definitions_fn: Optional[
+            Callable[[], List[RepositoryLevelDefinition]]
+        ] = lazy_definitions_fn or (lambda: [])
+        self._lazy_definitions: Optional[List[RepositoryLevelDefinition]] = None
+
+        self._all_definitions: Optional[List[RepositoryLevelDefinition]] = None
+
+    def _get_lazy_definitions(self):
+        if self._lazy_definitions is None:
+            self._lazy_definitions = self._lazy_definitions_fn()
+            for definition in self._lazy_definitions:
+                self._validate_and_cache_definition(definition, definition.name)
+
+        return self._lazy_definitions
+
+    def get_definition_names(self) -> List[str]:
         if self._definition_names:
             return self._definition_names
 
-        self._definition_names = list(self._definitions.keys())
+        self._definition_names = list(self._definitions.keys()) + [
+            definition.name for definition in self._get_lazy_definitions()
+        ]
         return self._definition_names
 
-    def has_definition(self, definition_name):
+    def has_definition(self, definition_name: str) -> bool:
         check.str_param(definition_name, "definition_name")
 
         return definition_name in self.get_definition_names()
 
-    def get_all_definitions(self):
+    def get_all_definitions(self) -> List[RepositoryLevelDefinition]:
         if self._all_definitions is not None:
             return self._all_definitions
 
@@ -63,13 +110,10 @@ class _CacheingDefinitionIndex(object):
         )
         return self._all_definitions
 
-    def get_definition(self, definition_name):
+    def get_definition(self, definition_name: str) -> RepositoryLevelDefinition:
         check.str_param(definition_name, "definition_name")
 
-        if definition_name in self._definition_cache:
-            return self._definition_cache[definition_name]
-
-        if definition_name not in self._definitions:
+        if not self.has_definition(definition_name):
             raise DagsterInvariantViolationError(
                 "Could not find {definition_kind} '{definition_name}'. Found: "
                 "{found_names}.".format(
@@ -84,38 +128,46 @@ class _CacheingDefinitionIndex(object):
                 )
             )
 
+        if definition_name in self._definition_cache:
+            return self._definition_cache[definition_name]
+
         definition_source = self._definitions[definition_name]
 
         if isinstance(definition_source, self._definition_class):
-            self._definition_cache[definition_name] = definition_source
+            self._definition_cache[definition_name] = self._validation_fn(definition_source)
             return definition_source
         else:
-            definition = definition_source()
-            check.invariant(
-                isinstance(definition, self._definition_class),
-                "Bad constructor for {definition_kind} {definition_name}: must return "
-                "{definition_class_name}, got value of type {type_}".format(
-                    definition_kind=self._definition_kind,
-                    definition_name=definition_name,
-                    definition_class_name=self._definition_class_name,
-                    type_=type(definition),
-                ),
-            )
-            check.invariant(
-                definition.name == definition_name,
-                "Bad constructor for {definition_kind} '{definition_name}': name in "
-                "{definition_class_name} does not match: got '{definition_def_name}'".format(
-                    definition_kind=self._definition_kind,
-                    definition_name=definition_name,
-                    definition_class_name=self._definition_class_name,
-                    definition_def_name=definition.name,
-                ),
-            )
-            self._definition_cache[definition_name] = definition
+            definition = cast(Callable, definition_source)()
+            self._validate_and_cache_definition(definition, definition_name)
             return definition
 
+    def _validate_and_cache_definition(
+        self, definition: RepositoryLevelDefinition, definition_dict_key: str
+    ):
+        check.invariant(
+            isinstance(definition, self._definition_class),
+            "Bad constructor for {definition_kind} {definition_name}: must return "
+            "{definition_class_name}, got value of type {type_}".format(
+                definition_kind=self._definition_kind,
+                definition_name=definition_dict_key,
+                definition_class_name=self._definition_class_name,
+                type_=type(definition),
+            ),
+        )
+        check.invariant(
+            definition.name == definition_dict_key,
+            "Bad constructor for {definition_kind} '{definition_name}': name in "
+            "{definition_class_name} does not match: got '{definition_def_name}'".format(
+                definition_kind=self._definition_kind,
+                definition_name=definition_dict_key,
+                definition_class_name=self._definition_class_name,
+                definition_def_name=definition.name,
+            ),
+        )
+        self._definition_cache[definition_dict_key] = self._validation_fn(definition)
 
-class RepositoryData(object):
+
+class RepositoryData:
     """Contains definitions belonging to a repository.
 
     Users should usually rely on the :py:func:`@repository <repository>` decorator to create new
@@ -124,7 +176,7 @@ class RepositoryData(object):
     of repository members.
     """
 
-    def __init__(self, pipelines, partition_sets, schedules, triggered_executions):
+    def __init__(self, pipelines, partition_sets, schedules, sensors):
         """Constructs a new RepositoryData object.
 
         You may pass pipeline, partition_set, and schedule definitions directly, or you may pass
@@ -143,26 +195,42 @@ class RepositoryData(object):
                 The partition sets belonging to the repository.
             schedules (Dict[str, Union[ScheduleDefinition, Callable[[], ScheduleDefinition]]]):
                 The schedules belonging to the repository.
-            triggered_executions (Dict[str, Union[TriggeredExecutionDefinition, Callable[[], TriggeredExecutionDefinition]]]):
-                The triggered executions for a repository.
+            sensors (Dict[str, Union[SensorDefinition, Callable[[], SensorDefinition]]]):
+                The sensors belonging to a repository.
 
         """
         check.dict_param(pipelines, "pipelines", key_type=str)
         check.dict_param(partition_sets, "partition_sets", key_type=str)
         check.dict_param(schedules, "schedules", key_type=str)
-        check.dict_param(triggered_executions, "triggered_executions", key_type=str)
+        check.dict_param(sensors, "sensors", key_type=str)
 
         self._pipelines = _CacheingDefinitionIndex(
-            PipelineDefinition, "PipelineDefinition", "pipeline", pipelines
+            PipelineDefinition, "PipelineDefinition", "pipeline", pipelines, self._validate_pipeline
         )
+
         self._schedules = _CacheingDefinitionIndex(
-            ScheduleDefinition, "ScheduleDefinition", "schedule", schedules
+            ScheduleDefinition,
+            "ScheduleDefinition",
+            "schedule",
+            schedules,
+            self._validate_schedule,
         )
         schedule_partition_sets = [
             schedule.get_partition_set()
             for schedule in self._schedules.get_all_definitions()
             if isinstance(schedule, PartitionScheduleDefinition)
         ]
+
+        def load_partition_sets_from_pipelines():
+            mode_partition_sets = []
+            for pipeline in self.get_all_pipelines():
+                for mode_def in pipeline.mode_definitions:
+                    partition_set_def = mode_def.get_partition_set_def(pipeline.name)
+                    if partition_set_def:
+                        mode_partition_sets.append(partition_set_def)
+
+            return mode_partition_sets
+
         self._partition_sets = _CacheingDefinitionIndex(
             PartitionSetDefinition,
             "PartitionSetDefinition",
@@ -171,13 +239,19 @@ class RepositoryData(object):
                 {partition_set.name: partition_set for partition_set in schedule_partition_sets},
                 partition_sets,
             ),
+            self._validate_partition_set,
+            load_partition_sets_from_pipelines,
         )
-        self._triggered_executions = _CacheingDefinitionIndex(
-            TriggeredExecutionDefinition,
-            "TriggeredExecutionDefinition",
-            "triggered_execution",
-            triggered_executions,
+        self._sensors = _CacheingDefinitionIndex(
+            SensorDefinition,
+            "SensorDefinition",
+            "sensor",
+            sensors,
+            self._validate_sensor,
         )
+        # load all sensors to force validation
+        self._sensors.get_all_definitions()
+
         self._all_pipelines = None
         self._solids = None
         self._all_solids = None
@@ -220,6 +294,14 @@ class RepositoryData(object):
             if key not in repository_definitions:
                 repository_definitions[key] = {}
 
+        duplicate_keys = set(repository_definitions.get("schedules", {}).keys()).intersection(
+            set(repository_definitions.get("sensors", {}).keys())
+        )
+        if duplicate_keys:
+            raise DagsterInvalidDefinitionError(
+                f"Duplicate definitions found for keys: {duplicate_keys.join(', ')}"
+            )
+
         return RepositoryData(**repository_definitions)
 
     @classmethod
@@ -234,10 +316,11 @@ class RepositoryData(object):
         pipelines = {}
         partition_sets = {}
         schedules = {}
-        triggered_executions = {}
+        sensors = {}
+        jobs = {}
         for definition in repository_definitions:
             if isinstance(definition, PipelineDefinition):
-                if definition.name in pipelines:
+                if definition.name in pipelines and pipelines[definition.name] != definition:
                     raise DagsterInvalidDefinitionError(
                         "Duplicate pipeline definition found for pipeline {pipeline_name}".format(
                             pipeline_name=definition.name
@@ -251,14 +334,27 @@ class RepositoryData(object):
                         "{partition_set_name}".format(partition_set_name=definition.name)
                     )
                 partition_sets[definition.name] = definition
-            elif isinstance(definition, ScheduleDefinition):
-                if definition.name in schedules:
+            elif isinstance(definition, SensorDefinition):
+                if definition.name in jobs:
                     raise DagsterInvalidDefinitionError(
-                        "Duplicate schedule definition found for schedule {schedule_name}".format(
-                            schedule_name=definition.name
-                        )
+                        f"Duplicate definition found for {definition.name}"
                     )
+                jobs[definition.name] = definition
+                sensors[definition.name] = definition
+                if definition.has_loadable_target():
+                    target = definition.load_target()
+                    pipelines[target.name] = target
+            elif isinstance(definition, ScheduleDefinition):
+                if definition.name in jobs:
+                    raise DagsterInvalidDefinitionError(
+                        f"Duplicate definition found for {definition.name}"
+                    )
+                jobs[definition.name] = definition
                 schedules[definition.name] = definition
+                if definition.has_loadable_target():
+                    target = definition.load_target()
+                    pipelines[target.name] = target
+
                 if isinstance(definition, PartitionScheduleDefinition):
                     partition_set_def = definition.get_partition_set()
                     if (
@@ -270,19 +366,18 @@ class RepositoryData(object):
                             "{partition_set_name}".format(partition_set_name=partition_set_def.name)
                         )
                     partition_sets[partition_set_def.name] = partition_set_def
-            elif isinstance(definition, TriggeredExecutionDefinition):
-                if definition.name in triggered_executions:
-                    raise DagsterInvalidDefinitionError(
-                        "Duplicate triggered execution definition found for triggered execution "
-                        "{name}".format(name=definition.name)
-                    )
-                triggered_executions[definition.name] = definition
+            elif isinstance(definition, GraphDefinition):
+                # error experience when this fails can be improved
+                coerced = definition.to_job(resource_defs={})
+                pipelines[coerced.name] = coerced
+            else:
+                check.failed(f"Unexpected repository entry {definition}")
 
         return RepositoryData(
             pipelines=pipelines,
             partition_sets=partition_sets,
             schedules=schedules,
-            triggered_executions=triggered_executions,
+            sensors=sensors,
         )
 
     def get_pipeline_names(self):
@@ -424,16 +519,14 @@ class RepositoryData(object):
 
         return self._schedules.has_definition(schedule_name)
 
-    def get_all_triggered_executions(self):
-        return self._triggered_executions.get_all_definitions()
+    def get_all_sensors(self):
+        return self._sensors.get_all_definitions()
 
-    def get_triggered_execution(self, trigger_name):
-        check.str_param(trigger_name, "trigger_name")
-        return self._triggered_executions.get_definition(trigger_name)
+    def get_sensor(self, name):
+        return self._sensors.get_definition(name)
 
-    def has_triggered_execution(self, trigger_name):
-        check.str_param(trigger_name, "trigger_name")
-        return self._triggered_executions.has_definition(trigger_name)
+    def has_sensor(self, name):
+        return self._sensors.has_definition(name)
 
     def get_all_solid_defs(self):
         if self._all_solids is not None:
@@ -496,8 +589,39 @@ class RepositoryData(object):
 
         return self._all_solids[name]
 
+    def _validate_pipeline(self, pipeline):
+        return pipeline
 
-class RepositoryDefinition(object):
+    def _validate_schedule(self, schedule):
+        pipelines = self.get_pipeline_names()
+
+        if schedule.pipeline_name not in pipelines:
+            raise DagsterInvalidDefinitionError(
+                f'ScheduleDefinition "{schedule.name}" targets pipeline "{schedule.pipeline_name}" '
+                "which was not found in this repository."
+            )
+
+        return schedule
+
+    def _validate_sensor(self, sensor):
+        pipelines = self.get_pipeline_names()
+        if sensor.pipeline_name is None:
+            # skip validation when the sensor does not target a pipeline
+            return sensor
+
+        if sensor.pipeline_name not in pipelines:
+            raise DagsterInvalidDefinitionError(
+                f'SensorDefinition "{sensor.name}" targets pipeline "{sensor.pipeline_name}" '
+                "which was not found in this repository."
+            )
+
+        return sensor
+
+    def _validate_partition_set(self, partition_set):
+        return partition_set
+
+
+class RepositoryDefinition:
     """Define a repository that contains a collection of definitions.
 
     Users should typically not create objects of this class directly. Instead, use the
@@ -510,9 +634,12 @@ class RepositoryDefinition(object):
     """
 
     def __init__(
-        self, name, repository_data, description=None,
+        self,
+        name,
+        repository_data,
+        description=None,
     ):
-        self._name = check.str_param(name, "name")
+        self._name = check_valid_name(name)
         self._description = check.opt_str_param(description, "description")
         self._repository_data = check.inst_param(repository_data, "repository_data", RepositoryData)
 
@@ -604,11 +731,16 @@ class RepositoryDefinition(object):
         return self._repository_data.has_schedule(name)
 
     @property
-    def triggered_execution_defs(self):
-        return self._repository_data.get_all_triggered_executions()
+    def sensor_defs(self):
+        return self._repository_data.get_all_sensors()
 
-    def get_triggered_execution_def(self, name):
-        return self._repository_data.get_triggered_execution(name)
+    def get_sensor_def(self, name):
+        return self._repository_data.get_sensor(name)
 
-    def has_triggered_execution_def(self, name):
-        return self._repository_data.has_triggered_execution(name)
+    def has_sensor_def(self, name):
+        return self._repository_data.has_sensor(name)
+
+    # If definition comes from the @repository decorator, then the __call__ method will be
+    # overwritten. Therefore, we want to maintain the call-ability of repository definitions.
+    def __call__(self, *args, **kwargs):
+        return self
