@@ -3,33 +3,29 @@ from abc import ABC, abstractmethod, abstractproperty
 from typing import TYPE_CHECKING, Any, Dict, Iterator, List, NamedTuple, Optional, Set, Union, cast
 
 from dagster import check
-from dagster.core.definitions import (
-    Failure,
-    InputDefinition,
-    PipelineDefinition,
-    RetryRequested,
-    SolidHandle,
-)
+from dagster.core.definitions import InputDefinition, PipelineDefinition, SolidHandle
 from dagster.core.definitions.events import AssetLineageInfo
 from dagster.core.errors import (
     DagsterExecutionLoadInputError,
+    DagsterInvariantViolationError,
     DagsterTypeLoadingError,
     user_code_error_boundary,
 )
 from dagster.core.storage.io_manager import IOManager
-from dagster.core.system_config.objects import EnvironmentConfig
+from dagster.core.system_config.objects import ResolvedRunConfig
 from dagster.serdes import whitelist_for_serdes
 from dagster.utils import ensure_gen
 
 from .objects import TypeCheckData
 from .outputs import StepOutputHandle, UnresolvedStepOutputHandle
-from .utils import build_resources_for_manager
+from .utils import build_resources_for_manager, solid_execution_error_boundary
 
 if TYPE_CHECKING:
     from dagster.core.types.dagster_type import DagsterType
     from dagster.core.storage.input_manager import InputManager
     from dagster.core.events import DagsterEvent
-    from dagster.core.execution.context.system import SystemStepExecutionContext, InputContext
+    from dagster.core.execution.context.system import StepExecutionContext
+    from dagster.core.execution.context.input import InputContext
 
 
 def _get_asset_lineage_from_fns(
@@ -115,15 +111,13 @@ class StepInputSource(ABC):
         pass
 
     @abstractmethod
-    def load_input_object(self, step_context: "SystemStepExecutionContext"):
+    def load_input_object(self, step_context: "StepExecutionContext"):
         raise NotImplementedError()
 
     def required_resource_keys(self, _pipeline_def: PipelineDefinition) -> Set[str]:
         return set()
 
-    def get_asset_lineage(
-        self, _step_context: "SystemStepExecutionContext"
-    ) -> List[AssetLineageInfo]:
+    def get_asset_lineage(self, _step_context: "StepExecutionContext") -> List[AssetLineageInfo]:
         return []
 
     @abstractmethod
@@ -131,12 +125,13 @@ class StepInputSource(ABC):
         self,
         step_versions: Dict[str, Optional[str]],
         pipeline_def: PipelineDefinition,
-        environment_config: EnvironmentConfig,
+        resolved_run_config: ResolvedRunConfig,
     ) -> Optional[str]:
         """See resolve_step_versions in resolve_versions.py for explanation of step_versions"""
         raise NotImplementedError()
 
 
+@whitelist_for_serdes
 class FromRootInputManager(
     NamedTuple(
         "_FromRootInputManager",
@@ -144,14 +139,12 @@ class FromRootInputManager(
     ),
     StepInputSource,
 ):
-    def load_input_object(
-        self, step_context: "SystemStepExecutionContext"
-    ) -> Iterator["DagsterEvent"]:
+    def load_input_object(self, step_context: "StepExecutionContext") -> Iterator["DagsterEvent"]:
         from dagster.core.events import DagsterEvent
 
         input_def = self.get_input_def(step_context.pipeline_def)
 
-        solid_config = step_context.environment_config.solids.get(str(self.solid_handle))
+        solid_config = step_context.resolved_run_config.solids.get(str(self.solid_handle))
         config_data = solid_config.inputs.get(self.input_name) if solid_config else None
 
         loader = getattr(step_context.resources, input_def.root_manager_key)
@@ -160,7 +153,7 @@ class FromRootInputManager(
             config_data,
             metadata=input_def.metadata,
             dagster_type=input_def.dagster_type,
-            resource_config=step_context.environment_config.resources[
+            resource_config=step_context.resolved_run_config.resources[
                 input_def.root_manager_key
             ].config,
             resources=build_resources_for_manager(input_def.root_manager_key, step_context),
@@ -172,15 +165,17 @@ class FromRootInputManager(
             manager_key=input_def.root_manager_key,
         )
 
-    def compute_version(self, step_versions, pipeline_def, environment_config) -> Optional[str]:
-        # TODO: support versioning for root loaders
-        return None
+    def compute_version(self, step_versions, pipeline_def, resolved_run_config) -> Optional[str]:
+        raise DagsterInvariantViolationError(
+            "Root input managers are currently not supported with memoization."
+        )
 
     def required_resource_keys(self, pipeline_def: PipelineDefinition) -> Set[str]:
         input_def = self.get_input_def(pipeline_def)
         return {input_def.root_manager_key}
 
 
+@whitelist_for_serdes
 class FromStepOutput(
     NamedTuple(
         "_FromStepOutput",
@@ -214,14 +209,16 @@ class FromStepOutput(
     def step_output_handle_dependencies(self) -> List[StepOutputHandle]:
         return [self.step_output_handle]
 
-    def get_load_context(self, step_context: "SystemStepExecutionContext") -> "InputContext":
-        io_manager_key = step_context.execution_plan.get_manager_key(self.step_output_handle)
-        resource_config = step_context.environment_config.resources[io_manager_key].config
+    def get_load_context(self, step_context: "StepExecutionContext") -> "InputContext":
+        io_manager_key = step_context.execution_plan.get_manager_key(
+            self.step_output_handle, step_context.pipeline_def
+        )
+        resource_config = step_context.resolved_run_config.resources[io_manager_key].config
         resources = build_resources_for_manager(io_manager_key, step_context)
 
         input_def = self.get_input_def(step_context.pipeline_def)
 
-        solid_config = step_context.environment_config.solids.get(str(self.solid_handle))
+        solid_config = step_context.resolved_run_config.solids.get(str(self.solid_handle))
         config_data = solid_config.inputs.get(self.input_name) if solid_config else None
 
         return step_context.for_input_manager(
@@ -234,14 +231,14 @@ class FromStepOutput(
             resources,
         )
 
-    def load_input_object(
-        self, step_context: "SystemStepExecutionContext"
-    ) -> Iterator["DagsterEvent"]:
+    def load_input_object(self, step_context: "StepExecutionContext") -> Iterator["DagsterEvent"]:
         from dagster.core.events import DagsterEvent
         from dagster.core.storage.intermediate_storage import IntermediateStorageAdapter
 
         source_handle = self.step_output_handle
-        manager_key = step_context.execution_plan.get_manager_key(source_handle)
+        manager_key = step_context.execution_plan.get_manager_key(
+            source_handle, step_context.pipeline_def
+        )
         input_manager = step_context.get_io_manager(source_handle)
         check.invariant(
             isinstance(input_manager, IOManager),
@@ -267,7 +264,7 @@ class FromStepOutput(
         self,
         step_versions: Dict[str, Optional[str]],
         pipeline_def: PipelineDefinition,
-        environment_config: EnvironmentConfig,
+        resolved_run_config: ResolvedRunConfig,
     ) -> Optional[str]:
         if (
             self.step_output_handle.step_key not in step_versions
@@ -282,9 +279,7 @@ class FromStepOutput(
     def required_resource_keys(self, _pipeline_def: PipelineDefinition) -> Set[str]:
         return set()
 
-    def get_asset_lineage(
-        self, step_context: "SystemStepExecutionContext"
-    ) -> List[AssetLineageInfo]:
+    def get_asset_lineage(self, step_context: "StepExecutionContext") -> List[AssetLineageInfo]:
         source_handle = self.step_output_handle
         input_manager = step_context.get_io_manager(source_handle)
         load_context = self.get_load_context(step_context)
@@ -322,6 +317,7 @@ class FromStepOutput(
         return []
 
 
+@whitelist_for_serdes
 class FromConfig(
     NamedTuple("_FromConfig", [("solid_handle", SolidHandle), ("input_name", str)]),
     StepInputSource,
@@ -335,7 +331,7 @@ class FromConfig(
             input_name=input_name,
         )
 
-    def load_input_object(self, step_context: "SystemStepExecutionContext") -> Any:
+    def load_input_object(self, step_context: "StepExecutionContext") -> Any:
         with user_code_error_boundary(
             DagsterTypeLoadingError,
             msg_fn=lambda: (
@@ -345,7 +341,7 @@ class FromConfig(
         ):
             dagster_type = self.get_input_def(step_context.pipeline_def).dagster_type
 
-            solid_config = step_context.environment_config.solids.get(str(self.solid_handle))
+            solid_config = step_context.resolved_run_config.solids.get(str(self.solid_handle))
             config_data = solid_config.inputs.get(self.input_name) if solid_config else None
 
             return dagster_type.loader.construct_from_config_value(step_context, config_data)
@@ -362,9 +358,9 @@ class FromConfig(
         self,
         step_versions: Dict[str, Optional[str]],
         pipeline_def: PipelineDefinition,
-        environment_config: EnvironmentConfig,
+        resolved_run_config: ResolvedRunConfig,
     ) -> Optional[str]:
-        solid_config = environment_config.solids.get(str(self.solid_handle))
+        solid_config = resolved_run_config.solids.get(str(self.solid_handle))
         config_data = solid_config.inputs.get(self.input_name) if solid_config else None
 
         solid_def = pipeline_def.get_solid(self.solid_handle)
@@ -372,6 +368,7 @@ class FromConfig(
         return dagster_type.loader.compute_loaded_input_version(config_data)
 
 
+@whitelist_for_serdes
 class FromDefaultValue(
     NamedTuple(
         "_FromDefaultValue",
@@ -389,18 +386,19 @@ class FromDefaultValue(
             self.input_name
         )
 
-    def load_input_object(self, step_context: "SystemStepExecutionContext"):
+    def load_input_object(self, step_context: "StepExecutionContext"):
         return self._load_value(step_context.pipeline_def)
 
     def compute_version(
         self,
         step_versions: Dict[str, Optional[str]],
         pipeline_def: PipelineDefinition,
-        environment_config: EnvironmentConfig,
+        resolved_run_config: ResolvedRunConfig,
     ) -> Optional[str]:
         return join_and_hash(repr(self._load_value(pipeline_def)))
 
 
+@whitelist_for_serdes
 class FromMultipleSources(
     NamedTuple(
         "_FromMultipleSources",
@@ -481,17 +479,15 @@ class FromMultipleSources(
             resource_keys = resource_keys.union(source.required_resource_keys(pipeline_def))
         return resource_keys
 
-    def compute_version(self, step_versions, pipeline_def, environment_config) -> Optional[str]:
+    def compute_version(self, step_versions, pipeline_def, resolved_run_config) -> Optional[str]:
         return join_and_hash(
             *[
-                inner_source.compute_version(step_versions, pipeline_def, environment_config)
+                inner_source.compute_version(step_versions, pipeline_def, resolved_run_config)
                 for inner_source in self.sources
             ]
         )
 
-    def get_asset_lineage(
-        self, step_context: "SystemStepExecutionContext"
-    ) -> List[AssetLineageInfo]:
+    def get_asset_lineage(self, step_context: "StepExecutionContext") -> List[AssetLineageInfo]:
         return [
             relation
             for source in self.sources
@@ -500,14 +496,17 @@ class FromMultipleSources(
 
 
 def _load_input_with_input_manager(input_manager: "InputManager", context: "InputContext"):
-    with user_code_error_boundary(
+    from dagster.core.execution.context.system import StepExecutionContext
+
+    step_context = cast(StepExecutionContext, context.step_context)
+    with solid_execution_error_boundary(
         DagsterExecutionLoadInputError,
-        control_flow_exceptions=[Failure, RetryRequested],
         msg_fn=lambda: (
             f'Error occurred while loading input "{context.name}" of '
-            f'step "{context.step_context.step.key}":'
+            f'step "{step_context.step.key}":'
         ),
-        step_key=context.step_context.step.key,
+        step_context=step_context,
+        step_key=step_context.step.key,
         input_name=context.name,
     ):
         value = input_manager.load_input(context)
@@ -515,6 +514,7 @@ def _load_input_with_input_manager(input_manager: "InputManager", context: "Inpu
     return value
 
 
+@whitelist_for_serdes
 class FromPendingDynamicStepOutput(
     NamedTuple(
         "_FromPendingDynamicStepOutput",
@@ -580,6 +580,7 @@ class FromPendingDynamicStepOutput(
         return pipeline_def.get_solid(self.solid_handle).input_def_named(self.input_name)
 
 
+@whitelist_for_serdes
 class FromUnresolvedStepOutput(
     NamedTuple(
         "_FromUnresolvedStepOutput",
@@ -639,6 +640,7 @@ class FromUnresolvedStepOutput(
         return pipeline_def.get_solid(self.solid_handle).input_def_named(self.input_name)
 
 
+@whitelist_for_serdes
 class FromDynamicCollect(
     NamedTuple(
         "_FromDynamicCollect",
@@ -728,3 +730,11 @@ class UnresolvedCollectStepInput(NamedTuple):
         """Return StepOutputHandles with placeholders, unresolved step keys and None mapping keys"""
 
         return [self.source.get_step_output_handle_dep_with_placeholder()]
+
+
+StepInputSourceTypes = (
+    StepInputSource,
+    FromDynamicCollect,
+    FromUnresolvedStepOutput,
+    FromPendingDynamicStepOutput,
+)
