@@ -4,10 +4,11 @@ from typing import Dict
 
 from dagster import check
 from dagster.core.definitions.events import AssetKey
-from dagster.core.events.log import EventRecord
+from dagster.core.events import DagsterEventType
+from dagster.core.events.log import EventLogEntry
 from dagster.serdes import ConfigurableClass
 
-from .base import EventLogSequence, EventLogStorage, extract_asset_events_cursor
+from .base import EventLogRecord, EventLogStorage, EventsCursor, extract_asset_events_cursor
 
 
 class InMemoryEventLogStorage(EventLogStorage, ConfigurableClass):
@@ -18,14 +19,14 @@ class InMemoryEventLogStorage(EventLogStorage, ConfigurableClass):
     """
 
     def __init__(self, inst_data=None, preload=None):
-        self._logs = defaultdict(EventLogSequence)
+        self._logs = defaultdict(list)
         self._handlers = defaultdict(set)
         self._inst_data = inst_data
         self._asset_tags = defaultdict(dict)
         self._wiped_asset_keys = defaultdict(float)
         if preload:
             for payload in preload:
-                self._logs[payload.pipeline_run.run_id] = EventLogSequence(payload.event_list)
+                self._logs[payload.pipeline_run.run_id] = payload.event_list
 
         super().__init__()
 
@@ -41,21 +42,31 @@ class InMemoryEventLogStorage(EventLogStorage, ConfigurableClass):
     def from_config_value(cls, inst_data, config_value):
         return cls(inst_data)
 
-    def get_logs_for_run(self, run_id, cursor=-1):
+    def get_logs_for_run(self, run_id, cursor=-1, of_type=None):
         check.str_param(run_id, "run_id")
         check.int_param(cursor, "cursor")
         check.invariant(
             cursor >= -1,
             "Don't know what to do with negative cursor {cursor}".format(cursor=cursor),
         )
+        check.opt_inst_param(of_type, "of_type", DagsterEventType)
 
         cursor = cursor + 1
-        return self._logs[run_id][cursor:]
+        if of_type:
+            return list(
+                filter(
+                    lambda r: r.is_dagster_event
+                    and r.dagster_event.event_type_value == of_type.value,
+                    self._logs[run_id][cursor:],
+                )
+            )
+        else:
+            return self._logs[run_id][cursor:]
 
     def store_event(self, event):
-        check.inst_param(event, "event", EventRecord)
+        check.inst_param(event, "event", EventLogEntry)
         run_id = event.run_id
-        self._logs[run_id] = self._logs[run_id].append(event)
+        self._logs[run_id].append(event)
 
         if event.is_dagster_event and event.dagster_event.asset_key:
             materialization = event.dagster_event.step_materialization_data.materialization
@@ -74,7 +85,7 @@ class InMemoryEventLogStorage(EventLogStorage, ConfigurableClass):
         pass
 
     def wipe(self):
-        self._logs = defaultdict(EventLogSequence)
+        self._logs = defaultdict(list)
 
     def watch(self, run_id, _start_cursor, callback):
         self._handlers[run_id].add(callback)
@@ -87,6 +98,40 @@ class InMemoryEventLogStorage(EventLogStorage, ConfigurableClass):
     def is_persistent(self):
         return False
 
+    def get_event_records(
+        self,
+        after_cursor=None,
+        limit=None,
+        ascending=False,
+        of_type=None,
+    ):
+        check.opt_inst_param(after_cursor, "after_cursor", EventsCursor)
+        after_id = after_cursor.id if after_cursor else None
+
+        filtered_events = []
+
+        for records in self._logs.values():
+            filtered_events += list(
+                filter(
+                    lambda r: of_type is None
+                    or (r.is_dagster_event and r.dagster_event.event_type_value == of_type.value),
+                    records,
+                )
+            )
+
+        event_records = [
+            EventLogRecord(storage_id=event_id, event_log_entry=event)
+            for event_id, event in enumerate(filtered_events)
+            if after_id is None or event_id > after_id
+        ]
+
+        event_records = sorted(event_records, key=lambda x: x.storage_id, reverse=not ascending)
+
+        if limit:
+            event_records = event_records[:limit]
+
+        return event_records
+
     def has_asset_key(self, asset_key: AssetKey) -> bool:
         for records in self._logs.values():
             for record in records:
@@ -127,6 +172,7 @@ class InMemoryEventLogStorage(EventLogStorage, ConfigurableClass):
         limit=None,
         ascending=False,
         include_cursor=False,
+        before_timestamp=None,
         cursor=None,
     ):
         asset_events = []
@@ -149,6 +195,7 @@ class InMemoryEventLogStorage(EventLogStorage, ConfigurableClass):
             for event_id, event in enumerate(asset_events)
             if (after_cursor is None or event_id > after_cursor)
             and (before_cursor is None or event_id < before_cursor)
+            and (before_timestamp is None or event.timestamp < before_timestamp)
         ]
 
         events_with_ids = sorted(
