@@ -8,16 +8,17 @@ import pendulum
 import sqlalchemy as db
 from dagster import check, seven
 from dagster.core.assets import AssetDetails
-from dagster.core.definitions.events import AssetKey
+from dagster.core.definitions.events import AssetKey, AssetMaterialization
 from dagster.core.errors import DagsterEventLogInvalidForRun
 from dagster.core.events import DagsterEventType
-from dagster.core.events.log import EventRecord
+from dagster.core.events.log import EventLogEntry
 from dagster.core.execution.stats import RunStepKeyStatsSnapshot, StepEventStatus
 from dagster.serdes import deserialize_json_to_dagster_namedtuple, serialize_dagster_namedtuple
+from dagster.serdes.errors import DeserializationError
 from dagster.utils import datetime_as_float, utc_datetime_from_naive, utc_datetime_from_timestamp
 
 from ..pipeline_run import PipelineRunStatsSnapshot
-from .base import EventLogStorage, extract_asset_events_cursor
+from .base import EventLogRecord, EventLogStorage, EventsCursor, extract_asset_events_cursor
 from .migration import REINDEX_DATA_MIGRATIONS
 from .schema import AssetKeyTable, SecondaryIndexMigrationTable, SqlEventLogStorageTable
 
@@ -89,26 +90,35 @@ class SqlEventLogStorage(EventLogStorage):
         )
 
     def store_asset(self, event):
-        check.inst_param(event, "event", EventRecord)
+        check.inst_param(event, "event", EventLogEntry)
         if not event.is_dagster_event or not event.dagster_event.asset_key:
             return
 
-        materialization = event.dagster_event.step_materialization_data.materialization
+        # We switched to storing the entire event record of the last materialization instead of just
+        # the AssetMaterialization object, so that we have access to metadata like timestamp,
+        # pipeline, run_id, etc.
+        #
+        # This should make certain asset queries way more performant, without having to do extra
+        # queries against the event log.
+        #
+        # This should be accompanied by a schema change in 0.12.0, renaming `last_materialization`
+        # to `last_materialization_event`, for clarity.  For now, we should do some back-compat.
+        #
+        # https://github.com/dagster-io/dagster/issues/3945
+
         with self.index_connection() as conn:
             try:
                 conn.execute(
                     AssetKeyTable.insert().values(  # pylint: disable=no-value-for-parameter
                         asset_key=event.dagster_event.asset_key.to_string(),
-                        last_materialization=serialize_dagster_namedtuple(materialization),
-                        last_run_id=event.run_id,
+                        last_materialization=serialize_dagster_namedtuple(event),
                     )
                 )
             except db.exc.IntegrityError:
                 conn.execute(
                     AssetKeyTable.update()  # pylint: disable=no-value-for-parameter
                     .values(
-                        last_materialization=serialize_dagster_namedtuple(materialization),
-                        last_run_id=event.run_id,
+                        last_materialization=serialize_dagster_namedtuple(event),
                     )
                     .where(
                         AssetKeyTable.c.asset_key == event.dagster_event.asset_key.to_string(),
@@ -119,9 +129,9 @@ class SqlEventLogStorage(EventLogStorage):
         """Store an event corresponding to a pipeline run.
 
         Args:
-            event (EventRecord): The event to store.
+            event (EventLogEntry): The event to store.
         """
-        check.inst_param(event, "event", EventRecord)
+        check.inst_param(event, "event", EventLogEntry)
         insert_event_statement = self.prepare_insert_event(event)
         run_id = event.run_id
 
@@ -131,23 +141,27 @@ class SqlEventLogStorage(EventLogStorage):
         if event.is_dagster_event and event.dagster_event.asset_key:
             self.store_asset(event)
 
-    def get_logs_for_run_by_log_id(self, run_id, cursor=-1):
+    def get_logs_for_run_by_log_id(self, run_id, cursor=-1, dagster_event_type=None):
         check.str_param(run_id, "run_id")
         check.int_param(cursor, "cursor")
         check.invariant(
             cursor >= -1,
             "Don't know what to do with negative cursor {cursor}".format(cursor=cursor),
         )
-
-        # cursor starts at 0 & auto-increment column starts at 1 so adjust
-        cursor = cursor + 1
+        check.opt_inst_param(dagster_event_type, "dagster_event_type", DagsterEventType)
 
         query = (
             db.select([SqlEventLogStorageTable.c.id, SqlEventLogStorageTable.c.event])
             .where(SqlEventLogStorageTable.c.run_id == run_id)
-            .where(SqlEventLogStorageTable.c.id > cursor)
             .order_by(SqlEventLogStorageTable.c.id.asc())
         )
+        if dagster_event_type:
+            query = query.where(
+                SqlEventLogStorageTable.c.dagster_event_type == dagster_event_type.value
+            )
+
+        # adjust 0 based index cursor to SQL offset
+        query = query.offset(cursor + 1)
 
         with self.run_connection(run_id) as conn:
             results = conn.execute(query).fetchall()
@@ -159,20 +173,21 @@ class SqlEventLogStorage(EventLogStorage):
                 json_str,
             ) in results:
                 events[record_id] = check.inst_param(
-                    deserialize_json_to_dagster_namedtuple(json_str), "event", EventRecord
+                    deserialize_json_to_dagster_namedtuple(json_str), "event", EventLogEntry
                 )
-        except (seven.JSONDecodeError, check.CheckError) as err:
+        except (seven.JSONDecodeError, DeserializationError) as err:
             raise DagsterEventLogInvalidForRun(run_id=run_id) from err
 
         return events
 
-    def get_logs_for_run(self, run_id, cursor=-1):
+    def get_logs_for_run(self, run_id, cursor=-1, of_type=None):
         """Get all of the logs corresponding to a run.
 
         Args:
             run_id (str): The id of the run for which to fetch logs.
             cursor (Optional[int]): Zero-indexed logs will be returned starting from cursor + 1,
                 i.e., if cursor is -1, all logs will be returned. (default: -1)
+            of_type (Optional[DagsterEventType]): the dagster event type to filter the logs.
         """
         check.str_param(run_id, "run_id")
         check.int_param(cursor, "cursor")
@@ -180,8 +195,9 @@ class SqlEventLogStorage(EventLogStorage):
             cursor >= -1,
             "Don't know what to do with negative cursor {cursor}".format(cursor=cursor),
         )
+        check.opt_inst_param(of_type, "of_type", DagsterEventType)
 
-        events_by_id = self.get_logs_for_run_by_log_id(run_id, cursor)
+        events_by_id = self.get_logs_for_run_by_log_id(run_id, cursor, of_type)
         return [event for id, event in sorted(events_by_id.items(), key=lambda x: x[0])]
 
     def get_stats_for_run(self, run_id):
@@ -233,7 +249,7 @@ class SqlEventLogStorage(EventLogStorage):
                 start_time=datetime_as_float(start_time) if start_time else None,
                 end_time=datetime_as_float(end_time) if end_time else None,
             )
-        except (seven.JSONDecodeError, check.CheckError) as err:
+        except (seven.JSONDecodeError, DeserializationError) as err:
             raise DagsterEventLogInvalidForRun(run_id=run_id) from err
 
     def get_step_stats_for_run(self, run_id, step_keys=None):
@@ -333,7 +349,7 @@ class SqlEventLogStorage(EventLogStorage):
         try:
             for (json_str,) in results:
                 event = check.inst_param(
-                    deserialize_json_to_dagster_namedtuple(json_str), "event", EventRecord
+                    deserialize_json_to_dagster_namedtuple(json_str), "event", EventLogEntry
                 )
                 if event.dagster_event.event_type == DagsterEventType.ASSET_MATERIALIZATION:
                     materializations[event.step_key].append(
@@ -343,7 +359,7 @@ class SqlEventLogStorage(EventLogStorage):
                     expectation_results[event.step_key].append(
                         event.dagster_event.event_specific_data.expectation_result
                     )
-        except (seven.JSONDecodeError, check.CheckError) as err:
+        except (seven.JSONDecodeError, DeserializationError) as err:
             raise DagsterEventLogInvalidForRun(run_id=run_id) from err
 
         return [
@@ -445,7 +461,7 @@ class SqlEventLogStorage(EventLogStorage):
     def update_event_log_record(self, record_id, event):
         """ Utility method for migration scripts to update SQL representation of event records. """
         check.int_param(record_id, "record_id")
-        check.inst_param(event, "event", EventRecord)
+        check.inst_param(event, "event", EventLogEntry)
         dagster_event_type = None
         asset_key_str = None
         if event.is_dagster_event:
@@ -461,7 +477,7 @@ class SqlEventLogStorage(EventLogStorage):
                 .values(
                     event=serialize_dagster_namedtuple(event),
                     dagster_event_type=dagster_event_type,
-                    timestamp=utc_datetime_from_timestamp(event.timestamp),
+                    timestamp=datetime.utcfromtimestamp(event.timestamp),
                     step_key=event.step_key,
                     asset_key=asset_key_str,
                 )
@@ -469,7 +485,7 @@ class SqlEventLogStorage(EventLogStorage):
 
     def get_event_log_table_data(self, run_id, record_id):
         """Utility method to test representation of the record in the SQL table.  Returns all of
-        the columns stored in the event log storage (as opposed to the deserialized `EventRecord`).
+        the columns stored in the event log storage (as opposed to the deserialized `EventLogEntry`).
         This allows checking that certain fields are extracted to support performant lookups (e.g.
         extracting `step_key` for fast filtering)"""
         with self.run_connection(run_id=run_id) as conn:
@@ -516,7 +532,13 @@ class SqlEventLogStorage(EventLogStorage):
                 )
 
     def _add_cursor_limit_to_query(
-        self, query, before_cursor, after_cursor, limit, ascending=False
+        self,
+        query,
+        before_cursor,
+        after_cursor,
+        limit,
+        ascending=False,
+        before_timestamp=None,
     ):
         """ Helper function to deal with cursor/limit pagination args """
 
@@ -530,6 +552,10 @@ class SqlEventLogStorage(EventLogStorage):
                 SqlEventLogStorageTable.c.id == after_cursor
             )
             query = query.where(SqlEventLogStorageTable.c.id > after_query)
+        if before_timestamp:
+            query = query.where(
+                SqlEventLogStorageTable.c.timestamp < datetime.utcfromtimestamp(before_timestamp)
+            )
 
         if limit:
             query = query.limit(limit)
@@ -541,6 +567,57 @@ class SqlEventLogStorage(EventLogStorage):
 
         return query
 
+    def get_event_records(
+        self,
+        after_cursor=None,
+        limit=None,
+        ascending=False,
+        of_type=None,
+    ):
+        """Returns a list of (record_id, record)."""
+
+        check.opt_inst_param(after_cursor, "after_cursor", EventsCursor)
+        check.opt_int_param(limit, "limit")
+        check.bool_param(ascending, "ascending")
+        check.opt_inst_param(of_type, "of_type", DagsterEventType)
+
+        after_id = after_cursor.id if after_cursor else None
+
+        query = db.select([SqlEventLogStorageTable.c.id, SqlEventLogStorageTable.c.event])
+        if of_type:
+            query = query.where(SqlEventLogStorageTable.c.dagster_event_type == of_type.value)
+
+        query = self._add_cursor_limit_to_query(
+            query=query,
+            before_cursor=None,
+            after_cursor=after_id,
+            limit=limit,
+            ascending=ascending,
+        )
+
+        with self.index_connection() as conn:
+            results = conn.execute(query).fetchall()
+
+        event_records = []
+        for row_id, json_str in results:
+            try:
+                event_record = deserialize_json_to_dagster_namedtuple(json_str)
+                if not isinstance(event_record, EventLogEntry):
+                    logging.warning(
+                        "Could not resolve event record as EventLogEntry for id `{}`.".format(
+                            row_id
+                        )
+                    )
+                    continue
+                else:
+                    event_records.append(
+                        EventLogRecord(storage_id=row_id, event_log_entry=event_record)
+                    )
+            except seven.JSONDecodeError:
+                logging.warning("Could not parse event record id `{}`.".format(row_id))
+
+        return event_records
+
     def has_asset_key(self, asset_key: AssetKey) -> bool:
         check.inst_param(asset_key, "asset_key", AssetKey)
         query = (
@@ -655,7 +732,7 @@ class SqlEventLogStorage(EventLogStorage):
 
         return query.where(
             SqlEventLogStorageTable.c.timestamp
-            > utc_datetime_from_timestamp(asset_details.last_wipe_timestamp)
+            > datetime.utcfromtimestamp(asset_details.last_wipe_timestamp)
         )
 
     def get_asset_events(
@@ -667,6 +744,7 @@ class SqlEventLogStorage(EventLogStorage):
         limit=None,
         ascending=False,
         include_cursor=False,
+        before_timestamp=None,
         cursor=None,  # deprecated
     ):
         check.inst_param(asset_key, "asset_key", AssetKey)
@@ -688,7 +766,12 @@ class SqlEventLogStorage(EventLogStorage):
         )
 
         query = self._add_cursor_limit_to_query(
-            query, before_cursor, after_cursor, limit, ascending=ascending
+            query,
+            before_cursor,
+            after_cursor,
+            limit,
+            before_timestamp=before_timestamp,
+            ascending=ascending,
         )
 
         with self.index_connection() as conn:
@@ -698,9 +781,9 @@ class SqlEventLogStorage(EventLogStorage):
         for row_id, json_str in results:
             try:
                 event_record = deserialize_json_to_dagster_namedtuple(json_str)
-                if not isinstance(event_record, EventRecord):
+                if not isinstance(event_record, EventLogEntry):
                     logging.warning(
-                        "Could not resolve asset event record as EventRecord for id `{}`.".format(
+                        "Could not resolve asset event record as EventLogEntry for id `{}`.".format(
                             row_id
                         )
                     )
@@ -739,16 +822,45 @@ class SqlEventLogStorage(EventLogStorage):
 
         return [run_id for (run_id, _timestamp) in results]
 
+    def _asset_materialization_from_json_column(self, json_str):
+        if not json_str:
+            return None
+
+        # We switched to storing the entire event record of the last materialization instead of just
+        # the AssetMaterialization object, so that we have access to metadata like timestamp,
+        # pipeline, run_id, etc.
+        #
+        # This should make certain asset queries way more performant, without having to do extra
+        # queries against the event log.
+        #
+        # This should be accompanied by a schema change in 0.12.0, renaming `last_materialization`
+        # to `last_materialization_event`, for clarity.  For now, we should do some back-compat.
+        #
+        # https://github.com/dagster-io/dagster/issues/3945
+
+        event_or_materialization = deserialize_json_to_dagster_namedtuple(json_str)
+        if isinstance(event_or_materialization, AssetMaterialization):
+            return event_or_materialization
+
+        if (
+            not isinstance(event_or_materialization, EventLogEntry)
+            or not event_or_materialization.is_dagster_event
+            or not event_or_materialization.dagster_event.asset_key
+        ):
+            return None
+
+        return event_or_materialization.dagster_event.step_materialization_data.materialization
+
     def all_asset_tags(self):
         query = db.select([AssetKeyTable.c.asset_key, AssetKeyTable.c.last_materialization])
         tags_by_asset_key = defaultdict(dict)
         with self.index_connection() as conn:
             rows = conn.execute(query).fetchall()
-            for asset_key, materialization_str in rows:
-                if materialization_str:
-                    materialization = deserialize_json_to_dagster_namedtuple(materialization_str)
+            for asset_key, json_str in rows:
+                materialization = self._asset_materialization_from_json_column(json_str)
+                if materialization and materialization.tags:
                     tags_by_asset_key[AssetKey.from_db_string(asset_key)] = {
-                        k: v for k, v in (materialization.tags or {}).items()
+                        k: v for k, v in materialization.tags.items()
                     }
 
         return tags_by_asset_key
@@ -763,8 +875,8 @@ class SqlEventLogStorage(EventLogStorage):
             if not rows or not rows[0] or not rows[0][0]:
                 return {}
 
-            materialization = deserialize_json_to_dagster_namedtuple(rows[0][0])
-            return materialization.tags or {}
+            materialization = self._asset_materialization_from_json_column(rows[0][0])
+            return materialization.tags if materialization and materialization.tags else {}
 
     def wipe_asset(self, asset_key):
         check.inst_param(asset_key, "asset_key", AssetKey)
