@@ -3,6 +3,7 @@ from collections import OrderedDict, defaultdict
 from dagster import check
 from dagster.core.errors import DagsterRunAlreadyExists, DagsterSnapshotDoesNotExist
 from dagster.core.events import DagsterEvent, DagsterEventType
+from dagster.core.execution.backfill import BulkActionStatus, PartitionBackfill
 from dagster.core.snap import (
     ExecutionPlanSnapshot,
     PipelineSnapshot,
@@ -16,8 +17,19 @@ from .base import RunStorage
 
 
 class InMemoryRunStorage(RunStorage):
-    def __init__(self):
+    def __init__(self, preload=None):
         self._init_storage()
+        if preload:
+            for payload in preload:
+                self._runs[payload.pipeline_run.run_id] = payload.pipeline_run
+                self._pipeline_snapshots[
+                    payload.pipeline_run.pipeline_snapshot_id
+                ] = payload.pipeline_snapshot
+                self._ep_snapshots[
+                    payload.pipeline_run.execution_plan_snapshot_id
+                ] = payload.execution_plan_snapshot
+
+        super().__init__()
 
     # separate method so it can be reused in wipe
     def _init_storage(self):
@@ -25,6 +37,7 @@ class InMemoryRunStorage(RunStorage):
         self._run_tags = defaultdict(dict)
         self._pipeline_snapshots = OrderedDict()
         self._ep_snapshots = OrderedDict()
+        self._bulk_actions = OrderedDict()
 
     def add_run(self, pipeline_run):
         check.inst_param(pipeline_run, "pipeline_run", PipelineRun)
@@ -49,6 +62,8 @@ class InMemoryRunStorage(RunStorage):
     def handle_run_event(self, run_id, event):
         check.str_param(run_id, "run_id")
         check.inst_param(event, "event", DagsterEvent)
+        if run_id not in self._runs:
+            return
         run = self._runs[run_id]
 
         if event.event_type == DagsterEventType.PIPELINE_START:
@@ -57,6 +72,14 @@ class InMemoryRunStorage(RunStorage):
             self._runs[run_id] = run.with_status(PipelineRunStatus.SUCCESS)
         elif event.event_type == DagsterEventType.PIPELINE_FAILURE:
             self._runs[run_id] = self._runs[run_id].with_status(PipelineRunStatus.FAILURE)
+        elif event.event_type == DagsterEventType.PIPELINE_ENQUEUED:
+            self._runs[run_id] = self._runs[run_id].with_status(PipelineRunStatus.QUEUED)
+        elif event.event_type == DagsterEventType.PIPELINE_STARTING:
+            self._runs[run_id] = self._runs[run_id].with_status(PipelineRunStatus.STARTING)
+        elif event.event_type == DagsterEventType.PIPELINE_CANCELING:
+            self._runs[run_id] = self._runs[run_id].with_status(PipelineRunStatus.CANCELING)
+        elif event.event_type == DagsterEventType.PIPELINE_CANCELED:
+            self._runs[run_id] = self._runs[run_id].with_status(PipelineRunStatus.CANCELED)
 
     def get_runs(self, filters=None, cursor=None, limit=None):
         check.opt_inst_param(filters, "filters", PipelineRunsFilter)
@@ -70,7 +93,7 @@ class InMemoryRunStorage(RunStorage):
             if filters.run_ids and run.run_id not in filters.run_ids:
                 return False
 
-            if filters.status and filters.status != run.status:
+            if filters.statuses and run.status not in filters.statuses:
                 return False
 
             if filters.pipeline_name and filters.pipeline_name != run.pipeline_name:
@@ -81,6 +104,9 @@ class InMemoryRunStorage(RunStorage):
             ):
                 return False
 
+            if filters.snapshot_id and filters.snapshot_id != run.pipeline_snapshot_id:
+                return False
+
             return True
 
         matching_runs = list(filter(run_filter, reversed(self._runs.values())))
@@ -91,10 +117,10 @@ class InMemoryRunStorage(RunStorage):
 
         return len(self.get_runs(filters))
 
-    def _slice(self, runs, cursor, limit):
+    def _slice(self, items, cursor, limit, key_fn=lambda _: _.run_id):
         if cursor:
             try:
-                index = next(i for i, run in enumerate(runs) if run.run_id == cursor)
+                index = next(i for i, item in enumerate(items) if key_fn(item) == cursor)
             except StopIteration:
                 return []
             start = index + 1
@@ -106,12 +132,15 @@ class InMemoryRunStorage(RunStorage):
         else:
             end = None
 
-        return list(runs)[start:end]
+        return list(items)[start:end]
 
     def get_run_by_id(self, run_id):
         check.str_param(run_id, "run_id")
         return self._runs.get(run_id)
 
+    def get_run_records(self, filters=None, limit=None, order_by=None, ascending=False):
+        raise NotImplementedError("In memory run storage does not track timestamp yet.")
+
     def get_run_tags(self):
         all_tags = defaultdict(set)
         for _run_id, tags in self._run_tags.items():
@@ -169,6 +198,9 @@ class InMemoryRunStorage(RunStorage):
     def wipe(self):
         self._init_storage()
 
+    def build_missing_indexes(self, print_fn=None, force_rebuild_all=False):
+        pass
+
     def get_run_group(self, run_id):
         check.str_param(run_id, "run_id")
         pipeline_run = self._runs.get(run_id)
@@ -208,3 +240,41 @@ class InMemoryRunStorage(RunStorage):
                 run_groups[root_run_id]["count"] += 1
 
         return run_groups
+
+    # Daemon Heartbeats
+
+    def add_daemon_heartbeat(self, daemon_heartbeat):
+        raise NotImplementedError(
+            "The dagster daemon lives in a separate process. It cannot use in memory storage."
+        )
+
+    def get_daemon_heartbeats(self):
+        raise NotImplementedError(
+            "The dagster daemon lives in a separate process. It cannot use in memory storage."
+        )
+
+    def wipe_daemon_heartbeats(self):
+        raise NotImplementedError(
+            "The dagster daemon lives in a separate process. It cannot use in memory storage."
+        )
+
+    def get_backfills(self, status=None, cursor=None, limit=None):
+        check.opt_inst_param(status, "status", BulkActionStatus)
+        backfills = [
+            backfill
+            for backfill in self._bulk_actions.values()
+            if not status or status == backfill.status
+        ]
+        return self._slice(backfills[::-1], cursor, limit, key_fn=lambda _: _.backfill_id)
+
+    def get_backfill(self, backfill_id):
+        check.str_param(backfill_id, "backfill_id")
+        return self._bulk_actions.get(backfill_id)
+
+    def add_backfill(self, partition_backfill):
+        check.inst_param(partition_backfill, "partition_backfill", PartitionBackfill)
+        self._bulk_actions[partition_backfill.backfill_id] = partition_backfill
+
+    def update_backfill(self, partition_backfill):
+        check.inst_param(partition_backfill, "partition_backfill", PartitionBackfill)
+        self._bulk_actions[partition_backfill.backfill_id] = partition_backfill
