@@ -1,7 +1,6 @@
 import re
 
 import pytest
-
 from dagster import (
     execute_pipeline,
     execute_pipeline_iterator,
@@ -9,7 +8,7 @@ from dagster import (
     reexecute_pipeline_iterator,
 )
 from dagster.core.definitions.pipeline_base import InMemoryPipeline
-from dagster.core.errors import DagsterInvalidSubsetError
+from dagster.core.errors import DagsterExecutionStepNotFoundError, DagsterInvalidSubsetError
 from dagster.core.instance import DagsterInstance
 from dagster.core.test_utils import step_output_event_filter
 
@@ -112,7 +111,7 @@ def test_execute_pipeline_iterator_with_solid_selection_query():
 
 def test_reexecute_pipeline_with_step_selection_single_clause():
     instance = DagsterInstance.ephemeral()
-    run_config = {"storage": {"filesystem": {}}}
+    run_config = {"intermediate_storage": {"filesystem": {}}}
     pipeline_result_full = execute_pipeline(foo_pipeline, run_config=run_config, instance=instance)
     assert pipeline_result_full.success
     assert pipeline_result_full.result_for_solid("add_one").output_value() == 7
@@ -134,7 +133,7 @@ def test_reexecute_pipeline_with_step_selection_single_clause():
         parent_run_id=pipeline_result_full.run_id,
         run_config=run_config,
         instance=instance,
-        step_selection=["*add_nums.compute"],
+        step_selection=["*add_nums"],
     )
 
     assert reexecution_result_up.success
@@ -145,7 +144,7 @@ def test_reexecute_pipeline_with_step_selection_single_clause():
         parent_run_id=pipeline_result_full.run_id,
         run_config=run_config,
         instance=instance,
-        step_selection=["add_nums.compute++"],
+        step_selection=["add_nums++"],
     )
     assert reexecution_result_down.success
     assert reexecution_result_down.result_for_solid("add_one").output_value() == 7
@@ -153,7 +152,7 @@ def test_reexecute_pipeline_with_step_selection_single_clause():
 
 def test_reexecute_pipeline_with_step_selection_multi_clauses():
     instance = DagsterInstance.ephemeral()
-    run_config = {"storage": {"filesystem": {}}}
+    run_config = {"intermediate_storage": {"filesystem": {}}}
     pipeline_result_full = execute_pipeline(foo_pipeline, run_config=run_config, instance=instance)
     assert pipeline_result_full.success
     assert pipeline_result_full.result_for_solid("add_one").output_value() == 7
@@ -164,7 +163,7 @@ def test_reexecute_pipeline_with_step_selection_multi_clauses():
         parent_run_id=pipeline_result_full.run_id,
         run_config=run_config,
         instance=instance,
-        step_selection=["return_one.compute", "return_two.compute", "add_nums.compute+"],
+        step_selection=["return_one", "return_two", "add_nums+"],
     )
     assert result_multi_disjoint.success
     assert result_multi_disjoint.result_for_solid("multiply_two").output_value() == 6
@@ -174,27 +173,39 @@ def test_reexecute_pipeline_with_step_selection_multi_clauses():
         parent_run_id=pipeline_result_full.run_id,
         run_config=run_config,
         instance=instance,
-        step_selection=["return_one.compute++", "return_two.compute", "add_nums.compute+"],
+        step_selection=["return_one++", "return_two", "add_nums+"],
     )
     assert result_multi_overlap.success
     assert result_multi_overlap.result_for_solid("multiply_two").output_value() == 6
 
     with pytest.raises(
-        DagsterInvalidSubsetError,
-        match=re.escape("No qualified steps to execute found for step_selection"),
+        DagsterExecutionStepNotFoundError,
+        match="Step selection refers to unknown step: a",
     ):
         reexecute_pipeline(
             foo_pipeline,
             parent_run_id=pipeline_result_full.run_id,
             run_config=run_config,
             instance=instance,
-            step_selection=["a", "*add_nums.compute"],
+            step_selection=["a", "*add_nums"],
+        )
+
+    with pytest.raises(
+        DagsterExecutionStepNotFoundError,
+        match="Step selection refers to unknown steps: a, b",
+    ):
+        reexecute_pipeline(
+            foo_pipeline,
+            parent_run_id=pipeline_result_full.run_id,
+            run_config=run_config,
+            instance=instance,
+            step_selection=["a+", "*b"],
         )
 
 
 def test_reexecute_pipeline_iterator():
     instance = DagsterInstance.ephemeral()
-    run_config = {"storage": {"filesystem": {}}}
+    run_config = {"intermediate_storage": {"filesystem": {}}}
     pipeline_result_full = execute_pipeline(foo_pipeline, run_config=run_config, instance=instance)
     assert pipeline_result_full.success
     assert pipeline_result_full.result_for_solid("add_one").output_value() == 7
@@ -217,7 +228,7 @@ def test_reexecute_pipeline_iterator():
             parent_run_id=pipeline_result_full.run_id,
             run_config=run_config,
             instance=instance,
-            step_selection=["*add_nums.compute"],
+            step_selection=["*add_nums"],
         )
     )
     events_up = list(output_event_iterator_up)
