@@ -3,6 +3,7 @@ import uuid
 import pytest
 from dagster import (
     DependencyDefinition,
+    Field,
     InputDefinition,
     Int,
     List,
@@ -18,12 +19,12 @@ from dagster import (
     check,
     execute_pipeline,
     execute_pipeline_iterator,
-    lambda_solid,
+    fs_io_manager,
     pipeline,
+    reconstructable,
     reexecute_pipeline,
     solid,
 )
-from dagster.cli.workspace.load import location_origin_from_python_file
 from dagster.core.definitions import Solid
 from dagster.core.definitions.dependency import DependencyStructure
 from dagster.core.definitions.graph import _create_adjacency_lists
@@ -37,6 +38,7 @@ from dagster.core.utility_solids import (
     define_stub_solid,
     input_set,
 )
+from dagster.core.workspace.load import location_origin_from_python_file
 from dagster.utils.test import execute_solid_within_pipeline
 
 # protected members
@@ -169,8 +171,7 @@ def test_external_diamond_toposort():
         python_file=__file__,
         attribute="create_diamond_pipeline",
         working_directory=None,
-    ).create_handle() as handle:
-        repo_location = handle.create_location()
+    ).create_test_location() as repo_location:
         external_repo = next(iter(repo_location.get_repositories().values()))
         external_pipeline = next(iter(external_repo.get_all_external_pipelines()))
         assert external_pipeline.solid_names_in_topological_order == [
@@ -330,11 +331,11 @@ def test_pipeline_name_threaded_through_context():
 
 
 def test_pipeline_subset():
-    @lambda_solid
+    @solid
     def return_one():
         return 1
 
-    @lambda_solid
+    @solid
     def add_one(num):
         return num + 1
 
@@ -360,11 +361,11 @@ def test_pipeline_subset():
 
 
 def test_pipeline_explicit_subset():
-    @lambda_solid
+    @solid
     def return_one():
         return 1
 
-    @lambda_solid
+    @solid
     def add_one(num):
         return num + 1
 
@@ -390,11 +391,11 @@ def test_pipeline_explicit_subset():
 
 
 def test_pipeline_subset_of_subset():
-    @lambda_solid
+    @solid
     def return_one():
         return 1
 
-    @lambda_solid
+    @solid
     def add_one(num):
         return num + 1
 
@@ -426,15 +427,15 @@ def test_pipeline_subset_of_subset():
 
 
 def test_pipeline_subset_with_multi_dependency():
-    @lambda_solid
+    @solid
     def return_one():
         return 1
 
-    @lambda_solid
+    @solid
     def return_two():
         return 2
 
-    @lambda_solid(input_defs=[InputDefinition("dep", Nothing)])
+    @solid(input_defs=[InputDefinition("dep", Nothing)])
     def noop():
         return 3
 
@@ -470,15 +471,15 @@ def test_pipeline_subset_with_multi_dependency():
 
 
 def test_pipeline_explicit_subset_with_multi_dependency():
-    @lambda_solid
+    @solid
     def return_one():
         return 1
 
-    @lambda_solid
+    @solid
     def return_two():
         return 2
 
-    @lambda_solid(input_defs=[InputDefinition("dep", Nothing)])
+    @solid(input_defs=[InputDefinition("dep", Nothing)])
     def noop():
         return 3
 
@@ -514,15 +515,15 @@ def test_pipeline_explicit_subset_with_multi_dependency():
 
 
 def define_three_part_pipeline():
-    @lambda_solid(input_defs=[InputDefinition("num", Int)], output_def=OutputDefinition(Int))
+    @solid(input_defs=[InputDefinition("num", Int)], output_defs=[OutputDefinition(Int)])
     def add_one(num):
         return num + 1
 
-    @lambda_solid(input_defs=[InputDefinition("num", Int)], output_def=OutputDefinition(Int))
+    @solid(input_defs=[InputDefinition("num", Int)], output_defs=[OutputDefinition(Int)])
     def add_two(num):
         return num + 2
 
-    @lambda_solid(input_defs=[InputDefinition("num", Int)], output_def=OutputDefinition(Int))
+    @solid(input_defs=[InputDefinition("num", Int)], output_defs=[OutputDefinition(Int)])
     def add_three(num):
         return num + 3
 
@@ -562,9 +563,9 @@ def test_pipeline_execution_explicit_disjoint_subset():
 
 
 def test_pipeline_wrapping_types():
-    @lambda_solid(
+    @solid(
         input_defs=[InputDefinition("value", Optional[List[Optional[String]]])],
-        output_def=OutputDefinition(Optional[List[Optional[String]]]),
+        output_defs=[OutputDefinition(Optional[List[Optional[String]]])],
     )
     def double_string_for_all(value):
         if not value:
@@ -605,12 +606,12 @@ def test_pipeline_wrapping_types():
 def test_pipeline_streaming_iterator():
     events = []
 
-    @lambda_solid
+    @solid
     def push_one():
         events.append(1)
         return 1
 
-    @lambda_solid
+    @solid
     def add_one(num):
         events.append(num + 1)
         return num + 1
@@ -688,11 +689,10 @@ def test_pipeline_init_failure():
         raise_on_error=False,
         instance=mem_instance,
     )
-
     assert result.success is False
     event = result.event_list[-1]
-    assert event.event_type_value == "PIPELINE_INIT_FAILURE"
-    assert event.pipeline_init_failure_data
+    assert event.event_type_value == "PIPELINE_FAILURE"
+    assert event.pipeline_failure_data
     assert mem_instance.get_run_by_id(result.run_id).is_failure
 
     with instance_for_test() as fs_instance:
@@ -704,17 +704,17 @@ def test_pipeline_init_failure():
         )
         assert result.success is False
         event = result.event_list[-1]
-        assert event.event_type_value == "PIPELINE_INIT_FAILURE"
-        assert event.pipeline_init_failure_data
+        assert event.event_type_value == "PIPELINE_FAILURE"
+        assert event.pipeline_failure_data
         assert fs_instance.get_run_by_id(result.run_id).is_failure
 
 
 def test_reexecution_fs_storage():
-    @lambda_solid
+    @solid
     def return_one():
         return 1
 
-    @lambda_solid
+    @solid
     def add_one(num):
         return num + 1
 
@@ -762,12 +762,80 @@ def test_reexecution_fs_storage():
     assert grandchild_run.root_run_id == pipeline_result.run_id
 
 
+def retry_pipeline():
+    @solid(
+        config_schema={
+            "fail": Field(bool, is_required=False, default_value=False),
+        },
+    )
+    def return_one(context):
+        if context.solid_config["fail"]:
+            raise Exception("FAILURE")
+        return 1
+
+    @solid
+    def add_one(num):
+        return num + 1
+
+    return PipelineDefinition(
+        solid_defs=[return_one, add_one],
+        name="test",
+        dependencies={"add_one": {"num": DependencyDefinition("return_one")}},
+        mode_defs=[ModeDefinition(resource_defs={"io_manager": fs_io_manager})],
+    )
+
+
+def test_multiproc_reexecution_fs_storage_after_fail():
+    with instance_for_test() as instance:
+        run_config = {"execution": {"multiprocess": {}}}
+        pipeline_result = execute_pipeline(
+            reconstructable(retry_pipeline),
+            run_config={
+                "execution": {"multiprocess": {}},
+                "solids": {"return_one": {"config": {"fail": True}}},
+            },
+            instance=instance,
+            raise_on_error=False,
+        )
+        assert not pipeline_result.success
+
+        reexecution_result = reexecute_pipeline(
+            reconstructable(retry_pipeline),
+            pipeline_result.run_id,
+            run_config=run_config,
+            instance=instance,
+        )
+
+        assert reexecution_result.success
+        assert len(reexecution_result.solid_result_list) == 2
+        assert reexecution_result.result_for_solid("return_one").output_value() == 1
+        assert reexecution_result.result_for_solid("add_one").output_value() == 2
+        reexecution_run = instance.get_run_by_id(reexecution_result.run_id)
+        assert reexecution_run.parent_run_id == pipeline_result.run_id
+        assert reexecution_run.root_run_id == pipeline_result.run_id
+
+        grandchild_result = reexecute_pipeline(
+            reconstructable(retry_pipeline),
+            reexecution_result.run_id,
+            run_config=run_config,
+            instance=instance,
+        )
+
+        assert grandchild_result.success
+        assert len(grandchild_result.solid_result_list) == 2
+        assert grandchild_result.result_for_solid("return_one").output_value() == 1
+        assert grandchild_result.result_for_solid("add_one").output_value() == 2
+        grandchild_run = instance.get_run_by_id(grandchild_result.run_id)
+        assert grandchild_run.parent_run_id == reexecution_result.run_id
+        assert grandchild_run.root_run_id == pipeline_result.run_id
+
+
 def test_reexecution_fs_storage_with_solid_selection():
-    @lambda_solid
+    @solid
     def return_one():
         return 1
 
-    @lambda_solid
+    @solid
     def add_one(num):
         return num + 1
 
@@ -826,7 +894,7 @@ def test_reexecution_fs_storage_with_solid_selection():
     #   re-exeucte a step which hasn't been included in the original pipeline
     with pytest.raises(
         DagsterExecutionStepNotFoundError,
-        match="Can not build subset plan from unknown step: add_one",
+        match="Step selection refers to unknown step: add_one",
     ):
         reexecute_pipeline(
             pipeline_def,
@@ -852,11 +920,11 @@ def test_reexecution_fs_storage_with_solid_selection():
 
 
 def test_single_step_reexecution():
-    @lambda_solid
+    @solid
     def return_one():
         return 1
 
-    @lambda_solid
+    @solid
     def add_one(num):
         return num + 1
 
@@ -886,11 +954,11 @@ def test_single_step_reexecution():
 
 
 def test_two_step_reexecution():
-    @lambda_solid
+    @solid
     def return_one():
         return 1
 
-    @lambda_solid
+    @solid
     def add_one(num):
         return num + 1
 
@@ -921,7 +989,7 @@ def test_optional():
     def return_optional(_context):
         yield Output(1, "x")
 
-    @lambda_solid
+    @solid
     def echo(x):
         return x
 
@@ -1022,11 +1090,11 @@ def test_pipeline_tags():
 
 
 def test_multi_dep_optional():
-    @lambda_solid
+    @solid
     def ret_one():
         return 1
 
-    @lambda_solid
+    @solid
     def echo(x):
         return x
 
