@@ -1,5 +1,7 @@
 import datetime
 import os
+import random
+import string
 import sys
 import tempfile
 import time
@@ -8,7 +10,11 @@ from contextlib import contextmanager
 import pendulum
 import pytest
 from dagster import (
+    Any,
     DagsterEventType,
+    Field,
+    Partition,
+    PartitionSetDefinition,
     ScheduleDefinition,
     daily_schedule,
     hourly_schedule,
@@ -18,8 +24,8 @@ from dagster import (
     seven,
     solid,
 )
-from dagster.core.definitions.job import RunRequest
 from dagster.core.definitions.reconstructable import ReconstructableRepository
+from dagster.core.definitions.run_request import RunRequest
 from dagster.core.errors import DagsterScheduleWipeRequired
 from dagster.core.host_representation import (
     ExternalJobOrigin,
@@ -37,9 +43,10 @@ from dagster.core.test_utils import (
     mock_system_timezone,
 )
 from dagster.core.types.loadable_target_origin import LoadableTargetOrigin
+from dagster.core.workspace.dynamic_workspace import DynamicWorkspace
 from dagster.daemon import get_default_daemon_logger
 from dagster.scheduler.scheduler import launch_scheduled_runs
-from dagster.seven import create_pendulum_time, to_timezone
+from dagster.seven.compat.pendulum import create_pendulum_time, to_timezone
 from dagster.utils import merge_dicts
 from dagster.utils.partitions import DEFAULT_DATE_FORMAT
 
@@ -152,12 +159,13 @@ def simple_temporary_schedule(date):
     return _solid_config(date)
 
 
-@daily_schedule(
+# forgot date arg
+@daily_schedule(  # type: ignore
     pipeline_name="the_pipeline",
     start_date=_COUPLE_DAYS_AGO,
     execution_timezone="UTC",
 )
-def bad_env_fn_schedule():  # forgot context arg
+def bad_env_fn_schedule():
     return {}
 
 
@@ -266,10 +274,73 @@ def the_other_repo():
     ]
 
 
+@solid(config_schema=Field(Any))
+def config_solid(_):
+    return 1
+
+
+@pipeline
+def config_pipeline():
+    config_solid()
+
+
+@daily_schedule(
+    pipeline_name="config_pipeline", start_date=_COUPLE_DAYS_AGO, execution_timezone="UTC"
+)
+def large_schedule(_):
+    REQUEST_CONFIG_COUNT = 120000
+
+    def _random_string(length):
+        return "".join(random.choice(string.ascii_lowercase) for x in range(length))
+
+    return {
+        "solids": {
+            "config_solid": {
+                "config": {
+                    "foo": {
+                        _random_string(10): _random_string(20) for i in range(REQUEST_CONFIG_COUNT)
+                    }
+                }
+            }
+        }
+    }
+
+
+@solid
+def start(_, x):
+    return x
+
+
+@solid
+def end(_, x=1):
+    return x
+
+
+@pipeline
+def two_step_pipeline():
+    end(start())
+
+
+manual_partition = PartitionSetDefinition(
+    name="manual_partition",
+    pipeline_name="two_step_pipeline",
+    # selects only second step
+    solid_selection=["end"],
+    partition_fn=lambda: ["one"],
+    # includes config for first step - test that it is ignored
+    run_config_fn_for_partition=lambda _: {"solids": {"start": {"inputs": {"x": {"value": 4}}}}},
+)
+
+manual_partition_schedule = manual_partition.create_schedule_definition(
+    "manual_partition_schedule", "0 0 * * *", lambda _x, _y: Partition("one")
+)
+
+
 @repository
 def the_repo():
     return [
         the_pipeline,
+        config_pipeline,
         simple_schedule,
         simple_temporary_schedule,
         simple_hourly_schedule,
@@ -288,6 +359,9 @@ def the_repo():
         define_multi_run_schedule(),
         define_multi_run_schedule_with_missing_run_key(),
         partitionless_schedule,
+        large_schedule,
+        two_step_pipeline,
+        manual_partition_schedule,
     ]
 
 
@@ -312,9 +386,10 @@ def logger():
 @contextmanager
 def instance_with_schedules(external_repo_context, overrides=None):
     with schedule_instance(overrides) as instance:
-        with ProcessGrpcServerRegistry(wait_for_processes_on_exit=True) as grpc_server_registry:
-            with external_repo_context() as external_repo:
-                yield (instance, grpc_server_registry, external_repo)
+        with ProcessGrpcServerRegistry() as grpc_server_registry:
+            with DynamicWorkspace(grpc_server_registry) as workspace:
+                with external_repo_context() as external_repo:
+                    yield (instance, workspace, external_repo)
 
 
 @contextmanager
@@ -328,8 +403,8 @@ def default_repo():
     with ManagedGrpcPythonEnvRepositoryLocationOrigin(
         loadable_target_origin=loadable_target_origin,
         location_name="test_location",
-    ).create_handle() as handle:
-        yield handle.create_location().get_repository("the_repo")
+    ).create_test_location() as location:
+        yield location.get_repository("the_repo")
 
 
 def repos():
@@ -402,7 +477,7 @@ def test_simple_schedule(external_repo_context, capfd):
     )
     with instance_with_schedules(external_repo_context) as (
         instance,
-        grpc_server_registry,
+        workspace,
         external_repo,
     ):
         with pendulum.test(freeze_datetime):
@@ -420,7 +495,7 @@ def test_simple_schedule(external_repo_context, capfd):
             list(
                 launch_scheduled_runs(
                     instance,
-                    grpc_server_registry,
+                    workspace,
                     logger(),
                     pendulum.now("UTC"),
                 )
@@ -443,7 +518,7 @@ def test_simple_schedule(external_repo_context, capfd):
             list(
                 launch_scheduled_runs(
                     instance,
-                    grpc_server_registry,
+                    workspace,
                     logger(),
                     pendulum.now("UTC"),
                 )
@@ -486,7 +561,7 @@ def test_simple_schedule(external_repo_context, capfd):
             list(
                 launch_scheduled_runs(
                     instance,
-                    grpc_server_registry,
+                    workspace,
                     logger(),
                     pendulum.now("UTC"),
                 )
@@ -502,7 +577,7 @@ def test_simple_schedule(external_repo_context, capfd):
             list(
                 launch_scheduled_runs(
                     instance,
-                    grpc_server_registry,
+                    workspace,
                     logger(),
                     pendulum.now("UTC"),
                 )
@@ -520,7 +595,7 @@ def test_simple_schedule(external_repo_context, capfd):
             list(
                 launch_scheduled_runs(
                     instance,
-                    grpc_server_registry,
+                    workspace,
                     logger(),
                     pendulum.now("UTC"),
                 )
@@ -547,9 +622,7 @@ def test_simple_schedule(external_repo_context, capfd):
             )
 
             # Check idempotence again
-            list(
-                launch_scheduled_runs(instance, grpc_server_registry, logger(), pendulum.now("UTC"))
-            )
+            list(launch_scheduled_runs(instance, workspace, logger(), pendulum.now("UTC")))
             assert instance.get_runs_count() == 3
             ticks = instance.get_job_ticks(schedule_origin.get_id())
             assert len(ticks) == 3
@@ -559,13 +632,13 @@ def test_simple_schedule(external_repo_context, capfd):
 def test_no_started_schedules(external_repo_context, capfd):
     with instance_with_schedules(external_repo_context) as (
         instance,
-        grpc_server_registry,
+        workspace,
         external_repo,
     ):
         external_schedule = external_repo.get_external_schedule("simple_schedule")
         schedule_origin = external_schedule.get_external_origin()
 
-        list(launch_scheduled_runs(instance, grpc_server_registry, logger(), pendulum.now("UTC")))
+        list(launch_scheduled_runs(instance, workspace, logger(), pendulum.now("UTC")))
         assert instance.get_runs_count() == 0
 
         ticks = instance.get_job_ticks(schedule_origin.get_id())
@@ -581,7 +654,7 @@ def test_schedule_without_timezone(external_repo_context, capfd):
     with mock_system_timezone("US/Eastern"):
         with instance_with_schedules(external_repo_context) as (
             instance,
-            grpc_server_registry,
+            workspace,
             external_repo,
         ):
             external_schedule = external_repo.get_external_schedule(
@@ -596,11 +669,7 @@ def test_schedule_without_timezone(external_repo_context, capfd):
 
                 instance.start_schedule_and_update_storage_state(external_schedule)
 
-                list(
-                    launch_scheduled_runs(
-                        instance, grpc_server_registry, logger(), pendulum.now("UTC")
-                    )
-                )
+                list(launch_scheduled_runs(instance, workspace, logger(), pendulum.now("UTC")))
 
                 assert instance.get_runs_count() == 1
 
@@ -633,11 +702,7 @@ def test_schedule_without_timezone(external_repo_context, capfd):
                 )
 
                 # Verify idempotence
-                list(
-                    launch_scheduled_runs(
-                        instance, grpc_server_registry, logger(), pendulum.now("UTC")
-                    )
-                )
+                list(launch_scheduled_runs(instance, workspace, logger(), pendulum.now("UTC")))
                 assert instance.get_runs_count() == 1
                 ticks = instance.get_job_ticks(schedule_origin.get_id())
                 assert len(ticks) == 1
@@ -647,7 +712,7 @@ def test_schedule_without_timezone(external_repo_context, capfd):
 def test_bad_env_fn(external_repo_context, capfd):
     with instance_with_schedules(external_repo_context) as (
         instance,
-        grpc_server_registry,
+        workspace,
         external_repo,
     ):
         external_schedule = external_repo.get_external_schedule("bad_env_fn_schedule")
@@ -658,9 +723,7 @@ def test_bad_env_fn(external_repo_context, capfd):
         with pendulum.test(initial_datetime):
             instance.start_schedule_and_update_storage_state(external_schedule)
 
-            list(
-                launch_scheduled_runs(instance, grpc_server_registry, logger(), pendulum.now("UTC"))
-            )
+            list(launch_scheduled_runs(instance, workspace, logger(), pendulum.now("UTC")))
 
             assert instance.get_runs_count() == 0
             ticks = instance.get_job_ticks(schedule_origin.get_id())
@@ -677,8 +740,6 @@ def test_bad_env_fn(external_repo_context, capfd):
 
             captured = capfd.readouterr()
 
-            assert "Failed to fetch schedule data for bad_env_fn_schedule: " in captured.out
-
             assert (
                 "Error occurred during the execution of run_config_fn for schedule bad_env_fn_schedule"
                 in captured.out
@@ -689,7 +750,7 @@ def test_bad_env_fn(external_repo_context, capfd):
 def test_bad_should_execute(external_repo_context, capfd):
     with instance_with_schedules(external_repo_context) as (
         instance,
-        grpc_server_registry,
+        workspace,
         external_repo,
     ):
         external_schedule = external_repo.get_external_schedule("bad_should_execute_schedule")
@@ -705,9 +766,7 @@ def test_bad_should_execute(external_repo_context, capfd):
         with pendulum.test(initial_datetime):
             instance.start_schedule_and_update_storage_state(external_schedule)
 
-            list(
-                launch_scheduled_runs(instance, grpc_server_registry, logger(), pendulum.now("UTC"))
-            )
+            list(launch_scheduled_runs(instance, workspace, logger(), pendulum.now("UTC")))
 
             assert instance.get_runs_count() == 0
             ticks = instance.get_job_ticks(schedule_origin.get_id())
@@ -723,9 +782,6 @@ def test_bad_should_execute(external_repo_context, capfd):
             )
 
             captured = capfd.readouterr()
-            assert (
-                "Failed to fetch schedule data for bad_should_execute_schedule: "
-            ) in captured.out
 
             assert (
                 "Error occurred during the execution of should_execute for schedule bad_should_execute_schedule"
@@ -739,7 +795,7 @@ def test_bad_should_execute(external_repo_context, capfd):
 def test_skip(external_repo_context, capfd):
     with instance_with_schedules(external_repo_context) as (
         instance,
-        grpc_server_registry,
+        workspace,
         external_repo,
     ):
         external_schedule = external_repo.get_external_schedule("skip_schedule")
@@ -751,9 +807,7 @@ def test_skip(external_repo_context, capfd):
         with pendulum.test(initial_datetime):
             instance.start_schedule_and_update_storage_state(external_schedule)
 
-            list(
-                launch_scheduled_runs(instance, grpc_server_registry, logger(), pendulum.now("UTC"))
-            )
+            list(launch_scheduled_runs(instance, workspace, logger(), pendulum.now("UTC")))
 
             assert instance.get_runs_count() == 0
             ticks = instance.get_job_ticks(schedule_origin.get_id())
@@ -780,7 +834,7 @@ def test_skip(external_repo_context, capfd):
 def test_wrong_config(external_repo_context, capfd):
     with instance_with_schedules(external_repo_context) as (
         instance,
-        grpc_server_registry,
+        workspace,
         external_repo,
     ):
         external_schedule = external_repo.get_external_schedule("wrong_config_schedule")
@@ -791,9 +845,7 @@ def test_wrong_config(external_repo_context, capfd):
         with pendulum.test(initial_datetime):
             instance.start_schedule_and_update_storage_state(external_schedule)
 
-            list(
-                launch_scheduled_runs(instance, grpc_server_registry, logger(), pendulum.now("UTC"))
-            )
+            list(launch_scheduled_runs(instance, workspace, logger(), pendulum.now("UTC")))
 
             assert instance.get_runs_count() == 1
 
@@ -853,7 +905,7 @@ def _get_unloadable_schedule_origin():
 def test_bad_schedules_mixed_with_good_schedule(external_repo_context, capfd):
     with instance_with_schedules(external_repo_context) as (
         instance,
-        grpc_server_registry,
+        workspace,
         external_repo,
     ):
         good_schedule = external_repo.get_external_schedule("simple_schedule")
@@ -886,9 +938,7 @@ def test_bad_schedules_mixed_with_good_schedule(external_repo_context, capfd):
             )
             instance.add_job_state(unloadable_schedule_state)
 
-            list(
-                launch_scheduled_runs(instance, grpc_server_registry, logger(), pendulum.now("UTC"))
-            )
+            list(launch_scheduled_runs(instance, workspace, logger(), pendulum.now("UTC")))
 
             assert instance.get_runs_count() == 1
             wait_for_all_runs_to_start(instance)
@@ -928,7 +978,7 @@ def test_bad_schedules_mixed_with_good_schedule(external_repo_context, capfd):
         initial_datetime = initial_datetime.add(days=1)
         with pendulum.test(initial_datetime):
             new_now = pendulum.now("UTC")
-            list(launch_scheduled_runs(instance, grpc_server_registry, logger(), new_now))
+            list(launch_scheduled_runs(instance, workspace, logger(), new_now))
 
             assert instance.get_runs_count() == 3
             wait_for_all_runs_to_start(instance)
@@ -985,7 +1035,7 @@ def test_bad_schedules_mixed_with_good_schedule(external_repo_context, capfd):
 def test_run_scheduled_on_time_boundary(external_repo_context):
     with instance_with_schedules(external_repo_context) as (
         instance,
-        grpc_server_registry,
+        workspace,
         external_repo,
     ):
         external_schedule = external_repo.get_external_schedule("simple_schedule")
@@ -1003,9 +1053,7 @@ def test_run_scheduled_on_time_boundary(external_repo_context):
             # Start schedule exactly at midnight
             instance.start_schedule_and_update_storage_state(external_schedule)
 
-            list(
-                launch_scheduled_runs(instance, grpc_server_registry, logger(), pendulum.now("UTC"))
-            )
+            list(launch_scheduled_runs(instance, workspace, logger(), pendulum.now("UTC")))
 
             assert instance.get_runs_count() == 1
             ticks = instance.get_job_ticks(schedule_origin.get_id())
@@ -1021,7 +1069,7 @@ def test_bad_load_repository(external_repo_context, capfd):
     )
     with instance_with_schedules(external_repo_context) as (
         instance,
-        grpc_server_registry,
+        workspace,
         external_repo,
     ):
         with pendulum.test(freeze_datetime):
@@ -1049,9 +1097,7 @@ def test_bad_load_repository(external_repo_context, capfd):
 
         initial_datetime = freeze_datetime.add(seconds=1)
         with pendulum.test(initial_datetime):
-            list(
-                launch_scheduled_runs(instance, grpc_server_registry, logger(), pendulum.now("UTC"))
-            )
+            list(launch_scheduled_runs(instance, workspace, logger(), pendulum.now("UTC")))
 
             assert instance.get_runs_count() == 0
 
@@ -1075,7 +1121,7 @@ def test_bad_load_schedule(external_repo_context, capfd):
     )
     with instance_with_schedules(external_repo_context) as (
         instance,
-        grpc_server_registry,
+        workspace,
         external_repo,
     ):
         with pendulum.test(freeze_datetime):
@@ -1100,9 +1146,7 @@ def test_bad_load_schedule(external_repo_context, capfd):
 
         initial_datetime = freeze_datetime.add(seconds=1)
         with pendulum.test(initial_datetime):
-            list(
-                launch_scheduled_runs(instance, grpc_server_registry, logger(), pendulum.now("UTC"))
-            )
+            list(launch_scheduled_runs(instance, workspace, logger(), pendulum.now("UTC")))
 
             assert instance.get_runs_count() == 0
 
@@ -1118,60 +1162,55 @@ def test_bad_load_schedule(external_repo_context, capfd):
 
 
 def test_bad_load_repository_location(capfd):
-    with schedule_instance() as instance, ProcessGrpcServerRegistry(
-        wait_for_processes_on_exit=True
-    ) as grpc_server_registry:
-        fake_origin = _get_unloadable_schedule_origin()
-        initial_datetime = create_pendulum_time(
-            year=2019,
-            month=2,
-            day=27,
-            hour=23,
-            minute=59,
-            second=59,
-        )
-        with pendulum.test(initial_datetime):
-            schedule_state = JobState(
-                fake_origin,
-                JobType.SCHEDULE,
-                JobStatus.RUNNING,
-                ScheduleJobData(
-                    "0 0 * * *", pendulum.now("UTC").timestamp(), "DagsterDaemonScheduler"
-                ),
+    with schedule_instance() as instance, ProcessGrpcServerRegistry() as grpc_server_registry:
+        with DynamicWorkspace(grpc_server_registry) as workspace:
+            fake_origin = _get_unloadable_schedule_origin()
+            initial_datetime = create_pendulum_time(
+                year=2019,
+                month=2,
+                day=27,
+                hour=23,
+                minute=59,
+                second=59,
             )
-            instance.add_job_state(schedule_state)
+            with pendulum.test(initial_datetime):
+                schedule_state = JobState(
+                    fake_origin,
+                    JobType.SCHEDULE,
+                    JobStatus.RUNNING,
+                    ScheduleJobData(
+                        "0 0 * * *", pendulum.now("UTC").timestamp(), "DagsterDaemonScheduler"
+                    ),
+                )
+                instance.add_job_state(schedule_state)
 
-        initial_datetime = initial_datetime.add(seconds=1)
-        with pendulum.test(initial_datetime):
-            list(
-                launch_scheduled_runs(instance, grpc_server_registry, logger(), pendulum.now("UTC"))
-            )
+            initial_datetime = initial_datetime.add(seconds=1)
+            with pendulum.test(initial_datetime):
+                list(launch_scheduled_runs(instance, workspace, logger(), pendulum.now("UTC")))
 
-            assert instance.get_runs_count() == 0
+                assert instance.get_runs_count() == 0
 
-            ticks = instance.get_job_ticks(fake_origin.get_id())
+                ticks = instance.get_job_ticks(fake_origin.get_id())
 
-            assert len(ticks) == 0
+                assert len(ticks) == 0
 
-            captured = capfd.readouterr()
-            assert "Scheduler caught an error for schedule doesnt_exist" in captured.out
-            assert "doesnt_exist not found at module scope" in captured.out
+                captured = capfd.readouterr()
+                assert "Scheduler caught an error for schedule doesnt_exist" in captured.out
+                assert "doesnt_exist not found at module scope" in captured.out
 
-        initial_datetime = initial_datetime.add(days=1)
-        with pendulum.test(initial_datetime):
-            list(
-                launch_scheduled_runs(instance, grpc_server_registry, logger(), pendulum.now("UTC"))
-            )
-            assert instance.get_runs_count() == 0
-            ticks = instance.get_job_ticks(fake_origin.get_id())
-            assert len(ticks) == 0
+            initial_datetime = initial_datetime.add(days=1)
+            with pendulum.test(initial_datetime):
+                list(launch_scheduled_runs(instance, workspace, logger(), pendulum.now("UTC")))
+                assert instance.get_runs_count() == 0
+                ticks = instance.get_job_ticks(fake_origin.get_id())
+                assert len(ticks) == 0
 
 
 @pytest.mark.parametrize("external_repo_context", repos())
 def test_multiple_schedules_on_different_time_ranges(external_repo_context, capfd):
     with instance_with_schedules(external_repo_context) as (
         instance,
-        grpc_server_registry,
+        workspace,
         external_repo,
     ):
         external_schedule = external_repo.get_external_schedule("simple_schedule")
@@ -1191,7 +1230,7 @@ def test_multiple_schedules_on_different_time_ranges(external_repo_context, capf
             list(
                 launch_scheduled_runs(
                     instance,
-                    grpc_server_registry,
+                    workspace,
                     logger(),
                     pendulum.now("UTC"),
                 )
@@ -1220,9 +1259,7 @@ def test_multiple_schedules_on_different_time_ranges(external_repo_context, capf
 
         initial_datetime = initial_datetime.add(hours=1)
         with pendulum.test(initial_datetime):
-            list(
-                launch_scheduled_runs(instance, grpc_server_registry, logger(), pendulum.now("UTC"))
-            )
+            list(launch_scheduled_runs(instance, workspace, logger(), pendulum.now("UTC")))
 
             assert instance.get_runs_count() == 3
 
@@ -1257,7 +1294,7 @@ def test_launch_failure(external_repo_context, capfd):
                 "class": "ExplodingRunLauncher",
             },
         },
-    ) as (instance, grpc_server_registry, external_repo):
+    ) as (instance, workspace, external_repo):
         external_schedule = external_repo.get_external_schedule("simple_schedule")
 
         schedule_origin = external_schedule.get_external_origin()
@@ -1269,9 +1306,7 @@ def test_launch_failure(external_repo_context, capfd):
         with pendulum.test(initial_datetime):
             instance.start_schedule_and_update_storage_state(external_schedule)
 
-            list(
-                launch_scheduled_runs(instance, grpc_server_registry, logger(), pendulum.now("UTC"))
-            )
+            list(launch_scheduled_runs(instance, workspace, logger(), pendulum.now("UTC")))
 
             assert instance.get_runs_count() == 1
 
@@ -1309,7 +1344,7 @@ def test_launch_failure(external_repo_context, capfd):
 
 def test_partitionless_schedule(capfd):
     initial_datetime = create_pendulum_time(year=2019, month=2, day=27, tz="US/Central")
-    with instance_with_schedules(default_repo) as (instance, grpc_server_registry, external_repo):
+    with instance_with_schedules(default_repo) as (instance, workspace, external_repo):
         with pendulum.test(initial_datetime):
             external_schedule = external_repo.get_external_schedule("partitionless_schedule")
             schedule_origin = external_schedule.get_external_origin()
@@ -1318,9 +1353,7 @@ def test_partitionless_schedule(capfd):
         # Travel enough in the future that many ticks have passed, but only one run executes
         initial_datetime = initial_datetime.add(days=5)
         with pendulum.test(initial_datetime):
-            list(
-                launch_scheduled_runs(instance, grpc_server_registry, logger(), pendulum.now("UTC"))
-            )
+            list(launch_scheduled_runs(instance, workspace, logger(), pendulum.now("UTC")))
             assert instance.get_runs_count() == 1
 
             wait_for_all_runs_to_start(instance)
@@ -1361,7 +1394,7 @@ def test_max_catchup_runs(capfd):
         create_pendulum_time(year=2019, month=2, day=27, hour=23, minute=59, second=59, tz="UTC"),
         "US/Central",
     )
-    with instance_with_schedules(default_repo) as (instance, grpc_server_registry, external_repo):
+    with instance_with_schedules(default_repo) as (instance, workspace, external_repo):
         with pendulum.test(initial_datetime):
             external_schedule = external_repo.get_external_schedule("simple_schedule")
             schedule_origin = external_schedule.get_external_origin()
@@ -1373,7 +1406,7 @@ def test_max_catchup_runs(capfd):
             list(
                 launch_scheduled_runs(
                     instance,
-                    grpc_server_registry,
+                    workspace,
                     logger(),
                     pendulum.now("UTC"),
                     max_catchup_runs=2,
@@ -1445,7 +1478,7 @@ def test_multi_runs(external_repo_context, capfd):
     )
     with instance_with_schedules(external_repo_context) as (
         instance,
-        grpc_server_registry,
+        workspace,
         external_repo,
     ):
         with pendulum.test(freeze_datetime):
@@ -1458,9 +1491,7 @@ def test_multi_runs(external_repo_context, capfd):
             assert len(ticks) == 0
 
             # launch_scheduled_runs does nothing before the first tick
-            list(
-                launch_scheduled_runs(instance, grpc_server_registry, logger(), pendulum.now("UTC"))
-            )
+            list(launch_scheduled_runs(instance, workspace, logger(), pendulum.now("UTC")))
             assert instance.get_runs_count() == 0
             ticks = instance.get_job_ticks(schedule_origin.get_id())
             assert len(ticks) == 0
@@ -1476,9 +1507,7 @@ def test_multi_runs(external_repo_context, capfd):
 
         freeze_datetime = freeze_datetime.add(seconds=2)
         with pendulum.test(freeze_datetime):
-            list(
-                launch_scheduled_runs(instance, grpc_server_registry, logger(), pendulum.now("UTC"))
-            )
+            list(launch_scheduled_runs(instance, workspace, logger(), pendulum.now("UTC")))
             assert instance.get_runs_count() == 2
             ticks = instance.get_job_ticks(schedule_origin.get_id())
             assert len(ticks) == 1
@@ -1511,9 +1540,7 @@ def test_multi_runs(external_repo_context, capfd):
             )
 
             # Verify idempotence
-            list(
-                launch_scheduled_runs(instance, grpc_server_registry, logger(), pendulum.now("UTC"))
-            )
+            list(launch_scheduled_runs(instance, workspace, logger(), pendulum.now("UTC")))
             assert instance.get_runs_count() == 2
             ticks = instance.get_job_ticks(schedule_origin.get_id())
             assert len(ticks) == 1
@@ -1524,9 +1551,7 @@ def test_multi_runs(external_repo_context, capfd):
             capfd.readouterr()
 
             # Traveling one more day in the future before running results in a tick
-            list(
-                launch_scheduled_runs(instance, grpc_server_registry, logger(), pendulum.now("UTC"))
-            )
+            list(launch_scheduled_runs(instance, workspace, logger(), pendulum.now("UTC")))
             assert instance.get_runs_count() == 4
             ticks = instance.get_job_ticks(schedule_origin.get_id())
             assert len(ticks) == 2
@@ -1552,7 +1577,7 @@ def test_multi_runs_missing_run_key(external_repo_context, capfd):
     )
     with instance_with_schedules(external_repo_context) as (
         instance,
-        grpc_server_registry,
+        workspace,
         external_repo,
     ):
         with pendulum.test(freeze_datetime):
@@ -1562,9 +1587,7 @@ def test_multi_runs_missing_run_key(external_repo_context, capfd):
             schedule_origin = external_schedule.get_external_origin()
             instance.start_schedule_and_update_storage_state(external_schedule)
 
-            list(
-                launch_scheduled_runs(instance, grpc_server_registry, logger(), pendulum.now("UTC"))
-            )
+            list(launch_scheduled_runs(instance, workspace, logger(), pendulum.now("UTC")))
             assert instance.get_runs_count() == 0
             ticks = instance.get_job_ticks(schedule_origin.get_id())
             assert len(ticks) == 1
@@ -1581,11 +1604,6 @@ def test_multi_runs_missing_run_key(external_repo_context, capfd):
 
             captured = capfd.readouterr()
 
-            assert (
-                "Failed to fetch schedule data for multi_run_schedule_with_missing_run_key: "
-                in captured.out
-            )
-
             assert (
                 "Error occurred during the execution function for schedule "
                 "multi_run_schedule_with_missing_run_key" in captured.out
@@ -1647,3 +1665,87 @@ def test_run_with_hanging_cron_schedules():
             },
         ):
             pass
+
+
+@pytest.mark.parametrize("external_repo_context", repos())
+def test_large_schedule(external_repo_context):
+    freeze_datetime = to_timezone(
+        create_pendulum_time(year=2019, month=2, day=27, hour=23, minute=59, second=59, tz="UTC"),
+        "US/Central",
+    )
+    with instance_with_schedules(external_repo_context) as (
+        instance,
+        workspace,
+        external_repo,
+    ):
+        with pendulum.test(freeze_datetime):
+            external_schedule = external_repo.get_external_schedule("large_schedule")
+            schedule_origin = external_schedule.get_external_origin()
+            instance.start_schedule_and_update_storage_state(external_schedule)
+
+            freeze_datetime = freeze_datetime.add(seconds=2)
+
+        with pendulum.test(freeze_datetime):
+            list(
+                launch_scheduled_runs(
+                    instance,
+                    workspace,
+                    logger(),
+                    pendulum.now("UTC"),
+                )
+            )
+
+            assert instance.get_runs_count() == 1
+            ticks = instance.get_job_ticks(schedule_origin.get_id())
+            assert len(ticks) == 1
+
+
+@pytest.mark.parametrize("external_repo_context", repos())
+def test_manual_partition_with_solid_selection(external_repo_context):
+    freeze_datetime = to_timezone(
+        create_pendulum_time(year=2019, month=2, day=27, hour=23, minute=59, second=59, tz="UTC"),
+        "US/Central",
+    )
+    with instance_with_schedules(external_repo_context) as (
+        instance,
+        workspace,
+        external_repo,
+    ):
+        with pendulum.test(freeze_datetime):
+            external_schedule = external_repo.get_external_schedule("manual_partition_schedule")
+            schedule_origin = external_schedule.get_external_origin()
+            instance.start_schedule_and_update_storage_state(external_schedule)
+
+            freeze_datetime = freeze_datetime.add(seconds=2)
+
+        with pendulum.test(freeze_datetime):
+            list(
+                launch_scheduled_runs(
+                    instance,
+                    workspace,
+                    logger(),
+                    pendulum.now("UTC"),
+                )
+            )
+
+            assert instance.get_runs_count() == 1
+            ticks = instance.get_job_ticks(schedule_origin.get_id())
+            assert len(ticks) == 1
+            run_id = ticks[0].run_ids[0]
+
+            start_time = time.time()
+            while (time.time() - start_time) < 5:
+                run = instance.get_run_by_id(run_id)
+                if not run.is_finished:
+                    time.sleep(0.1)
+                else:
+                    break
+
+            events = instance.all_logs(run_id)
+            started_steps = set()
+
+            for event in events:
+                if event.is_dagster_event and event.dagster_event.is_step_start:
+                    started_steps.add(event.dagster_event.step_key)
+
+            assert started_steps == {"end"}  # matches solid_selection
